package com.bocloud.cmp.driver;

import com.bocloud.cmp.driver.utils.VendorConverter;
import com.bocloud.cmp.entity.enums.CloudProvider;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.ButlerRepository;
import com.bocloud.cmp.provider.SuperProvider;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @Description：
 * @Author ：丁明威
 * @Date ：Created in 19:22 2018/11/14
 * @Modified By：
 */
public class GenericDriver<T extends SuperProvider> {
    private static Logger logger = LoggerFactory.getLogger(GenericDriver.class);
    private static String PREFIX = "bocloud.cmp.";
    ButlerRepository butlerRepository = ButlerRepository.getInstance();

    @SuppressWarnings("unchecked")
    public T getProvider(String vendor, String projectName, Class<?> clazz, ProviderRepository repository) {
        Butler butler = getButler(vendor);
        if (null == butler) {
            return null;
        }
        if (!StringUtils.isEmpty(projectName)) {
            butler.setProjectName(projectName);
        }
        String provider = getProvider(butler);
        return (T) repository.getProvider(provider, clazz, butler);
    }

    @SuppressWarnings("unchecked")
    public T getProvider(String vendor, Class<?> clazz, ProviderRepository repository) {
        Butler butler = getButler(vendor);
        if (null == butler) {
            return null;
        }
        String provider = getProvider(butler);
        return (T) repository.getProvider(provider, clazz, butler);
    }

    @SuppressWarnings("unchecked")
    public T getProvider(String vendor, Class<?> clazz, ProviderRepository repository, String region) {
        Butler butler = getButler(vendor);
        if (null == butler) {
            return null;
        }
        if (StringUtils.isNotEmpty(region) && !region.equalsIgnoreCase(butler.getRegionId())) {
            butler.setRegionId(region);
        }
        String provider = getProvider(butler);
        return (T) repository.getProvider(provider, clazz, butler);
    }

    @SuppressWarnings("unchecked")
    public T getProvider(String vendor, String username, String password, String projectId, String domainName,
                         Class<?> clazz, ProviderRepository repository) {
        Butler butler = getButler(vendor);
        if (null == butler) {
            return null;
        }
        butler.setUsername(username);
        butler.setPassword(password);
        butler.setProjectId(projectId);
        butler.setDomainName(domainName);
        String provider = getProvider(butler);
        return (T) repository.getProvider(provider, clazz, butler);
    }

    @SuppressWarnings("unchecked")
    public T getProvider(String vendor, String projectName, Class<?> clazz, ProviderRepository repository,
                         String region) {
        Butler butler = getButler(vendor);
        if (null == butler) {
            return null;
        }
        if (StringUtils.isNotEmpty(region) && !region.equalsIgnoreCase(butler.getRegionId())) {
            butler.setRegionId(region);
        }
        if (!StringUtils.isEmpty(projectName)) {
            butler.setProjectName(projectName);
            butler.setTenant(projectName);
        }
        String provider = getProvider(butler);
        return (T) repository.getProvider(provider, clazz, butler);
    }

    @SuppressWarnings("unchecked")
    public T getProvider(Butler butler, Class<?> clazz, ProviderRepository repository) {
        if (null == butler) {
            return null;
        }
        String provider = getProvider(butler);
        return (T) repository.getProvider(provider, clazz, butler);
    }

    public Result before(T provider) {
        if (null == provider) {
            return Result.FAILED("暂不支持！");
        }
        if (provider.accessable()) {
            return Result.SUCCESS("success");
        } else {
            return Result.FAILED("云平台认证失败!");
        }
    }

    public Butler getButler(String vendor) {
        Butler butler = butlerRepository.get(vendor);
        if (null == butler) {
            logger.error("no butler found of uuid {}", vendor);
            return null;
        }
        CloudProvider catalog = VendorConverter.convert(butler.getCategory());
        if (CloudProvider.UNKNOWN.equals(catalog)) {
            logger.warn("get unknown provider of {}", butler.getCategory());
            return null;
        } else if (CloudProvider.HCSO.equals(catalog)) {
            butler.setCategory(CloudProvider.HUAWEI.toString());
            return butler;
        } else {
            return butler;
        }
    }

    private String getProvider(Butler butler) {
        return getProviderValueByButler(butler);
    }

    /**
     * 根据Butler对象获取目标类的Provider注解上的value值
     *
     * @author sxp
     * @since 2023/5/26
     */
    public static String getProviderValueByButler(Butler butler) {
        logger.info("butler name={},category={},version={},url={}", butler.getName(), butler.getCategory(), butler.getVersion(), butler.getUrl());

        if (CloudProvider.HCSO.name().equalsIgnoreCase(butler.getCategory())) {
            return PREFIX + CloudProvider.HUAWEI.name().toLowerCase();
        }
        if (CloudProvider.SANGFOR.name().equalsIgnoreCase(butler.getCategory())) {
            if (butler.getVersion().equals("6.8")) {
                return PREFIX + CloudProvider.SANGFOR.name().toLowerCase() + ".v68";
            } else {
                return PREFIX + CloudProvider.SANGFOR.name().toLowerCase() + ".v63";
            }
        }
        return PREFIX + butler.getCategory().toLowerCase();
    }

}