package com.bocloud.cmp.driver.agile.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.agile.network.ExternalConnectProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.agile.AgileExternalConnectCidrModel;
import com.bocloud.nms.model.agile.AgileExternalConnectModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: ExternalConnectDriver
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/6/29 17:01
 */
@Service("externalConnectDriver")
public class ExternalConnectDriver extends GenericDriver<ExternalConnectProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult<AgileExternalConnectModel> create(String vendor, String region, AgileExternalConnectModel model) {
        ExternalConnectProvider provider = getProvider(vendor, ExternalConnectProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult<AgileExternalConnectModel> modify(String vendor, String region, AgileExternalConnectModel model) {
        ExternalConnectProvider provider = getProvider(vendor, ExternalConnectProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult<AgileExternalConnectModel> detail(String vendor, String region, String uuid) {
        ExternalConnectProvider provider = getProvider(vendor, ExternalConnectProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.detail(uuid);
    }

    public BocloudResult<?> remove(String vendor, String region, String uuid) {
        ExternalConnectProvider provider = getProvider(vendor, ExternalConnectProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.remove(uuid);
    }

    public BocloudResult<?> addCidr(String vendor, String region, AgileExternalConnectModel model, String cidrType) {
        ExternalConnectProvider provider = getProvider(vendor, ExternalConnectProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.addCidr(model, cidrType);
    }

    public BocloudResult<?> removeCidr(String vendor, String region, AgileExternalConnectModel model, String cidrType) {
        ExternalConnectProvider provider = getProvider(vendor, ExternalConnectProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.removeCidr(model, cidrType);
    }

    public BocloudResult<List<AgileExternalConnectCidrModel>> listCidr(String vendor, String region, String uuid) {
        ExternalConnectProvider provider = getProvider(vendor, ExternalConnectProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.listCidr(uuid);
    }

}