package com.bocloud.cmp.driver.azure.account;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.azure.account.BatchAccountModel;
import com.bocloud.cmp.provider.azure.account.BatchAccountProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: BatchAccountDriver
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/7/24 14:08
 */
@Service("batchAccountDriver")
public class BatchAccountDriver extends GenericDriver<BatchAccountProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult create(String vendor, String region, BatchAccountModel model) {
        BatchAccountProvider provider = getProvider(vendor, BatchAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult modify(String vendor, String region, BatchAccountModel model) {
        BatchAccountProvider provider = getProvider(vendor, BatchAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult changeStorageAccount(String vendor, String region, BatchAccountModel model) {
        BatchAccountProvider provider = getProvider(vendor, BatchAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.changeStorageAccount(model);
    }

    public BocloudResult detail(String vendor, String region, String uuid) {
        BatchAccountProvider provider = getProvider(vendor, BatchAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.detail(uuid);
    }

    public BocloudResult remove(String vendor, String region, String uuid) {
        BatchAccountProvider provider = getProvider(vendor, BatchAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(uuid);
    }

}