package com.bocloud.cmp.driver.azure.account;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.azure.account.StorageAccountFirewallModel;
import com.bocloud.cmp.model.azure.account.StorageAccountModel;
import com.bocloud.cmp.provider.azure.account.StorageAccountProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: StorageAccountDriver
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/7/24 14:08
 */

@Service("storageAccountDriver")
public class StorageAccountDriver extends GenericDriver<StorageAccountProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult create(String vendor, String region, StorageAccountModel model) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }


    public BocloudResult detail(String vendor, String region, String uuid) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.detail(uuid);
    }

    public BocloudResult checkName(String vendor, String region, String name) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.checkName(name);
    }

    public BocloudResult remove(String vendor, String region, String uuid) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(uuid);
    }

    public BocloudResult customDomain(String vendor, String region, StorageAccountModel model) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.customDomain(model);
    }

    public BocloudResult minimumTlsVersion(String vendor, String region, StorageAccountModel model) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.minimumTlsVersion(model);
    }

    public BocloudResult modify(String vendor, String region, StorageAccountModel model) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult withAccessFromNetworkSubnet(String vendor, String region, StorageAccountModel model) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.withAccessFromNetworkSubnet(model);
    }

    public BocloudResult withoutNetworkSubnetAccess(String vendor, String region, StorageAccountModel model) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.withoutNetworkSubnetAccess(model);
    }

    public BocloudResult withAccessFromAllNetworks(String vendor, String region, StorageAccountModel model) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.withAccessFromAllNetworks(model);
    }

    public BocloudResult withAccessFromIpAddress(String vendor, String region, StorageAccountFirewallModel model) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.withAccessFromIpAddress(model);
    }

    public BocloudResult withoutIpAddressAccess(String vendor, String region, StorageAccountFirewallModel model) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.withoutIpAddressAccess(model);
    }

    public BocloudResult regenerateKey(String vendor, String region, StorageAccountModel model) {
        StorageAccountProvider provider = getProvider(vendor, StorageAccountProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.regenerateKey(model);
    }

}