package com.bocloud.cmp.driver.azure.automation;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.azure.automation.AutomationAccountModel;
import com.bocloud.cmp.provider.azure.automation.AutomationAccoutProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AutomationAccountDriver
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/8/10 11:17
 */
@Service("automationAccountDriver")
public class AutomationAccountDriver extends GenericDriver<AutomationAccoutProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor) {
        AutomationAccoutProvider provider = getProvider(vendor, AutomationAccoutProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult detail(String vendor, AutomationAccountModel model) {
        AutomationAccoutProvider provider = getProvider(vendor, AutomationAccoutProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detail(model);
    }

    public BocloudResult create(String vendor, String region, AutomationAccountModel model) {
        AutomationAccoutProvider provider = getProvider(vendor, AutomationAccoutProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult remove(String vendor, AutomationAccountModel model) {
        AutomationAccoutProvider provider = getProvider(vendor, AutomationAccoutProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(model);
    }

    public BocloudResult getStatusCounts(String vendor, AutomationAccountModel model, String resourceId) {
        AutomationAccoutProvider provider = getProvider(vendor, AutomationAccoutProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.getStatusCounts(model, resourceId);
    }

    public BocloudResult softwareUpdateConfigurationMachineRuns(String vendor, AutomationAccountModel model, String resourceId) {
        AutomationAccoutProvider provider = getProvider(vendor, AutomationAccoutProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.softwareUpdateConfigurationMachineRuns(model, resourceId);
    }

    public BocloudResult hybridRunbookWorkerGroups(String vendor, AutomationAccountModel model, String resourceId) {
        AutomationAccoutProvider provider = getProvider(vendor, AutomationAccoutProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.hybridRunbookWorkerGroups(model, resourceId);
    }
}