package com.bocloud.cmp.driver.azure.automation;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.azure.automation.AutomationAccountModel;
import com.bocloud.cmp.model.azure.automation.DeploymentPlanModel;
import com.bocloud.cmp.provider.azure.automation.DeploymentProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: DeploymentDriver
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/8/11 18:04
 */
@Service("azureDeploymentDriver")
public class DeploymentDriver extends GenericDriver<DeploymentProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult listDeploymentPlan(String vendor, DeploymentPlanModel model) {
        DeploymentProvider provider = getProvider(vendor, DeploymentProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.listDeploymentPlan(model);
    }

    public BocloudResult createDeploymentPlan(String vendor, DeploymentPlanModel model) {
        DeploymentProvider provider = getProvider(vendor, DeploymentProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.createDeploymentPlan(model);
    }

    public BocloudResult removeDeploymentPlan(String vendor, DeploymentPlanModel model) {
        DeploymentProvider provider = getProvider(vendor, DeploymentProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.removeDeploymentPlan(model);
    }

    public BocloudResult deploymentAutomationAccount(String vendor, String regionId, AutomationAccountModel model, String deploymentName) {
        DeploymentProvider provider = getProvider(vendor, DeploymentProvider.class, repository, regionId);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.deploymentAutomationAccount(model, deploymentName);
    }

    public BocloudResult softwareUpdateConfigurationRuns(String vendor, String regionId, AutomationAccountModel model) {
        DeploymentProvider provider = getProvider(vendor, DeploymentProvider.class, repository, regionId);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.softwareUpdateConfigurationRuns(model);
    }

}