package com.bocloud.cmp.driver.azure.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.RouterModel;
import com.bocloud.cmp.model.azure.network.RouterTableModel;
import com.bocloud.cmp.provider.azure.network.RouterTableProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: RouterTableDriver
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/1/4 15:51
 */
@Service("routerTableDriver")
public class RouterTableDriver extends GenericDriver<RouterTableProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor) {
        RouterTableProvider provider = getProvider(vendor, RouterTableProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult detail(String vendor, String region, String uuid) {
        RouterTableProvider provider = getProvider(vendor, RouterTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detail(uuid);
    }

    public BocloudResult create(String vendor, String region, RouterTableModel model) {
        RouterTableProvider provider = getProvider(vendor, RouterTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult modify(String vendor, String region, RouterTableModel model) {
        RouterTableProvider provider = getProvider(vendor, RouterTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult remove(String vendor, String region, String uuid) {
        RouterTableProvider provider = getProvider(vendor, RouterTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(uuid);
    }

    //弃用
    public BocloudResult addRouter(String vendor, String region, RouterModel model) {
        RouterTableProvider provider = getProvider(vendor, RouterTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.addRouter(model);
    }

    //弃用
    public BocloudResult removeRouter(String vendor, String region, RouterModel model) {
        RouterTableProvider provider = getProvider(vendor, RouterTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.removeRouter(model);
    }
}