package com.bocloud.cmp.driver.blob;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.azure.account.StorageAccountModel;
import com.bocloud.cmp.provider.blob.BlobProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.OutputStream;

/**
 * 对象存储blob调用
 *
 * @author wangyu
 * @version 5.0.0-SNAPSHOT
 * @since 2021/2/1
 */
@Service
public class BlobDriver extends GenericDriver<BlobProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult upload(File file, String blobType, String standardBlobTier, String containerName, StorageAccountModel account, String vendor) {
        BlobProvider provider = getProvider(vendor, BlobProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.upload(file, blobType, standardBlobTier, containerName, account);
    }

    public BocloudResult download(OutputStream outputStream, String blobName, String containerName, StorageAccountModel account, String vendor) {
        BlobProvider provider = getProvider(vendor, BlobProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.download(outputStream, blobName, containerName, account);
    }

    public BocloudResult download(String filePath, String blobName, String containerName, StorageAccountModel account, String vendor) {
        BlobProvider provider = getProvider(vendor, BlobProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.download(filePath, blobName, containerName, account);
    }

    public BocloudResult delete(String blobName, String containerName, StorageAccountModel account, String vendor) {
        BlobProvider provider = getProvider(vendor, BlobProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.delete(blobName, containerName, account);
    }

    public BocloudResult createSnapshot(String blobName, String containerName, StorageAccountModel account, String vendor) {
        BlobProvider provider = getProvider(vendor, BlobProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.createSnapshot(blobName, containerName, account);
    }

    public BocloudResult deleteSnapshot(String snapshotName, String snapshotId, String containerName, StorageAccountModel account, String vendor) {
        BlobProvider provider = getProvider(vendor, BlobProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.deleteSnapshot(snapshotName, snapshotId, containerName, account);
    }

    public BocloudResult downloadSnapshot(String filePath, String snapshotName, String snapshotId, String containerName, StorageAccountModel account, String vendor) {
        BlobProvider provider = getProvider(vendor, BlobProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.downloadSnapshot(filePath, snapshotName, snapshotId, containerName, account);
    }

    public BocloudResult uploadStandardBlobTier(String blobOrSnapshotName, String snapshotId, String standardBlobTier, String containerName, StorageAccountModel account, String vendor) {
        BlobProvider provider = getProvider(vendor, BlobProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.uploadStandardBlobTier(blobOrSnapshotName, snapshotId, standardBlobTier, containerName, account);
    }
}
