package com.bocloud.cmp.driver.blob;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.azure.account.StorageAccountModel;
import com.bocloud.cmp.model.azure.blob.ContainerModel;
import com.bocloud.cmp.provider.blob.ContainerProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * 对象存储容器接口
 *
 * @author wangyu
 * @version 5.0.0-SNAPSHOT
 * @since 2021/2/1
 */
@Service
public class ContainerDriver extends GenericDriver<ContainerProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(StorageAccountModel account, String vendor) {
        ContainerProvider provider = getProvider(vendor, ContainerProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.list(account);
    }

    public BocloudResult create(ContainerModel model, StorageAccountModel account, String vendor) {
        ContainerProvider provider = getProvider(vendor, ContainerProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.create(model, account);
    }

    public BocloudResult delete(String name, StorageAccountModel account, String vendor) {
        ContainerProvider provider = getProvider(vendor, ContainerProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.delete(name, account);
    }

    public BocloudResult updateMetadata(String name, Map<String, String> metadata, StorageAccountModel account, String vendor) {
        ContainerProvider provider = getProvider(vendor, ContainerProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.updateMetadata(name, metadata, account);
    }

    public BocloudResult updatePublicAccess(String name, String publicAccessType, StorageAccountModel account, String vendor) {
        ContainerProvider provider = getProvider(vendor, ContainerProvider.class, repository, null);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.updatePublicAccess(name, publicAccessType, account);
    }
}
