package com.bocloud.cmp.driver.campus;


import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.campus.AcmUserProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.campus.AcmUserModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @description:
 * @author: Li Yaohui
 * @since: 2021/2/26 10:50 上午
 */
@Service("acmUserDriver")
public class AcmUserDriver extends GenericDriver<AcmUserProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult<List<AcmUserModel>> list(String vendor) {
        AcmUserProvider provider = getProvider(vendor, AcmUserProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult<AcmUserModel> create(String vendor, AcmUserModel model) {
        AcmUserProvider provider = getProvider(vendor, AcmUserProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult<AcmUserModel> getByName(String vendor, String username) {
        AcmUserProvider provider = getProvider(vendor, AcmUserProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.getByName(username);
    }

    public BocloudResult<?> modify(String vendor, AcmUserModel model) {
        AcmUserProvider provider = getProvider(vendor, AcmUserProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.modify(model);
    }

    /**
     * @param cancelType     注销类型：必须，缺省值为1，1：强制销户；2：结帐销户；3：退还服务周期费并销户，若输入除1、2、3以外的其他值，则按1处理
     * @param cancelPlatUser 是否注销平台用户:非必须，缺省值为0,1：注销平台用户；0：不注销平台用户，若输入除0,1以外的其他值，则按0处理。
     */
    public BocloudResult<?> delete(String vendor, String username, Integer cancelType, Integer cancelPlatUser) {
        AcmUserProvider provider = getProvider(vendor, AcmUserProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.delete(username, cancelType, cancelPlatUser);
    }

}
