package com.bocloud.cmp.driver.campus;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.campus.DeviceVlanProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.campus.DeviceVlanModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service("campusDeviceVlanDriver")
public class DeviceVlanDriver extends GenericDriver<DeviceVlanProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult<List<DeviceVlanModel>> list(String vendor, Long deviceId) {
        DeviceVlanProvider provider = getProvider(vendor, DeviceVlanProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.list(deviceId);
    }

    public BocloudResult<DeviceVlanModel> create(String vendor, Long deviceId, DeviceVlanModel model) {
        DeviceVlanProvider provider = getProvider(vendor, DeviceVlanProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.create(deviceId, model);
    }

    public BocloudResult<?> remove(String vendor, Long deviceId, Integer vlanUuid) {
        DeviceVlanProvider provider = getProvider(vendor, DeviceVlanProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.remove(deviceId, vlanUuid);
    }

    public BocloudResult<List<Long>> addInterfaceToVlan(String vendor, Long deviceId, Integer vlanId, List<Long> portIds) {
        DeviceVlanProvider provider = getProvider(vendor, DeviceVlanProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.addInterfaceToVlan(deviceId, vlanId, portIds);
    }

    public BocloudResult<List<Long>> removeInterfaceFromVlan(String vendor, Long deviceId, Integer vlanId, List<Long> portIds) {
        DeviceVlanProvider provider = getProvider(vendor, DeviceVlanProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.removeInterfaceFromVlan(deviceId, vlanId, portIds);
    }



}
