package com.bocloud.cmp.driver.campus;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.campus.GuestProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.campus.GuestAdminModel;
import com.bocloud.nms.model.campus.GuestUserModel;
import com.bocloud.nms.model.campus.OnlineUserModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service("guestDriver")
public class GuestDriver extends GenericDriver<GuestProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult<List<GuestUserModel>> listGuestUser(String vendor) {
        GuestProvider provider = getProvider(vendor, GuestProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.listGuestUser();
    }

    public BocloudResult<List<GuestAdminModel>> listGuestAdmin(String vendor) {
        GuestProvider provider = getProvider(vendor, GuestProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.listGuestAdmin();
    }

    public BocloudResult<List<OnlineUserModel>> listOnlineUser(String vendor) {
        GuestProvider provider = getProvider(vendor, GuestProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.listOnlineUser();
    }

    public BocloudResult<GuestUserModel> createGuestUser(String vendor, GuestUserModel model) {
        GuestProvider provider = getProvider(vendor, GuestProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.createGuestUser(model);
    }

    public BocloudResult<GuestAdminModel> createGuestAdmin(String vendor, GuestAdminModel model) {
        GuestProvider provider = getProvider(vendor, GuestProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.createGuestAdmin(model);
    }

    public BocloudResult<?> modifyGuestUser(String vendor, GuestUserModel model) {
        GuestProvider provider = getProvider(vendor, GuestProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.modifyGuestUser(model);
    }

    public BocloudResult<?> modifyGuestAdmin(String vendor, GuestAdminModel model) {
        GuestProvider provider = getProvider(vendor, GuestProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.modifyGuestAdmin(model);
    }

    public BocloudResult<?> removeGuestUser(String vendor, String userName) {
        GuestProvider provider = getProvider(vendor, GuestProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.removeGuestUser(userName);
    }

    public BocloudResult<?> removeGuestAdmin(String vendor, String userName) {
        GuestProvider provider = getProvider(vendor, GuestProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.removeGuestAdmin(userName);
    }

}
