package com.bocloud.cmp.driver.common;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.OSTenantModel;
import com.bocloud.cmp.provider.common.AuthProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 验证服务接口实现类
 *
 * @author weiwei
 */
@Service("authDriver")
public class AuthDriver extends GenericDriver<AuthProvider> {
	private static Logger logger = LoggerFactory.getLogger(AuthDriver.class);


	@Autowired
	private ProviderRepository repository;

	public BocloudResult auth(String vendor, String region) {
		AuthProvider provider = getProvider(vendor, AuthProvider.class, repository, region);
		Result result = before(provider);
		logger.info("before(provider) result is {}", JSON.toJSONString(result));

		if (result.isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.auth();
	}

	/**
	 * 从内存中删除缓存的用户验证信息
	 *
	 * @author sxp
	 * @since Nov 20, 2018
	 */
	public BocloudResult removeAuthFromMemory(String vendorUuid, String endpoint, String username) {
		AuthProvider provider = getProvider(vendorUuid, AuthProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeAuthFromMemory(endpoint, username);
	}

	public BocloudResult createProject(String vendorUuid, OSTenantModel model) {
		AuthProvider provider = getProvider(vendorUuid, AuthProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createProject(model);
	}

	public BocloudResult createProject(String vendorUuid, String projectName) {
		AuthProvider provider = getProvider(vendorUuid, AuthProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createProject(projectName);
	}

	public BocloudResult removeProject(String vendorUuid, String projectName) {
		AuthProvider provider = getProvider(vendorUuid, AuthProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeProject(projectName);
	}

	public BocloudResult modifyProject(String vendorUuid, OSTenantModel osModel) {
		AuthProvider provider = getProvider(vendorUuid, AuthProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.modifyProject(osModel);
	}

}
