package com.bocloud.cmp.driver.common;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.PricingModel;
import com.bocloud.cmp.provider.common.BillingProvider;
import com.bocloud.common.model.BocloudResult;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("billingDriver")
public class BillingDriver extends GenericDriver<BillingProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult billDetailList(String vendor, String month, String region) {
		BillingProvider provider = getProvider(vendor, BillingProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.billDetailList(month);
	}

	//---------------------------azure------------------------------------
	public BocloudResult billPeriods(String vendor) {
		BillingProvider provider = getProvider(vendor, BillingProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.billPeriods();
	}

	public BocloudResult periodPrices(String vendor, String periodUuid) {
		BillingProvider provider = getProvider(vendor, BillingProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.periodPrices(periodUuid);
	}

	public BocloudResult nextLinkPrices(String vendor, String nextLink) {
		BillingProvider provider = getProvider(vendor, BillingProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.nextLinkPrices(nextLink);
	}

	public BocloudResult periodByTimePrices(String vendor, String periodUuid, String startTime, String endTime) {
		BillingProvider provider = getProvider(vendor, BillingProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.periodByTimePrices(periodUuid, startTime, endTime);
	}

	public BocloudResult pricing(String vendor, PricingModel pricingModel) {
		BillingProvider provider = getProvider(vendor, BillingProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.pricing(pricingModel);
	}

	public BocloudResult pricing(String vendor, String regionId, PricingModel pricingModel) {
		BillingProvider provider = getProvider(vendor, BillingProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.pricing(pricingModel);
	}
}
