package com.bocloud.cmp.driver.common;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.common.FlavorProvider;
import com.bocloud.common.model.BocloudResult;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("flavorDriver")
public class FlavorDriver extends GenericDriver<FlavorProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult sync(String vendor, String region) {
		FlavorProvider provider = getProvider(vendor, FlavorProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.sync();
	}

	public BocloudResult create(String vendor, FlavorModel flavor) {
		FlavorProvider provider = getProvider(vendor, FlavorProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(flavor);
	}

	public BocloudResult create(String vendor, FlavorModel flavor, String username, String password, String projectId,
			String domainName) {
		FlavorProvider provider = getProvider(vendor, username, password, projectId, domainName, FlavorProvider.class,
				repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(flavor);
	}

	public BocloudResult create(String vendor, String projectName, FlavorModel flavor) {
		FlavorProvider provider = getProvider(vendor, projectName, FlavorProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(flavor);
	}

	public BocloudResult remove(String flavorId, String vendor) {
		FlavorProvider provider = getProvider(vendor, FlavorProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(flavorId);
	}

	public BocloudResult update(String vendor, FlavorModel flavor) {
		FlavorProvider provider = getProvider(vendor, FlavorProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(flavor);
	}

	public BocloudResult get(String vendor, String flavorUuid) {
		FlavorProvider provider = getProvider(vendor, FlavorProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.get(flavorUuid);
	}

	public BocloudResult<List<String>> listUsedFlavors(String vendor, String instanceId, String region) {
		FlavorProvider provider = getProvider(vendor, FlavorProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult<List<String>>(false, "云平台认证失败！");
		}
		return provider.listUsedFlavors(instanceId);
	}
}
