package com.bocloud.cmp.driver.common;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.DataStoreModel;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.LbListenerModel;
import com.bocloud.cmp.model.contentlibrary.LibraryModel;
import com.bocloud.cmp.provider.common.LocationProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 位置服务实现类
 *
 * @author dmw
 */
@Service("locationDriver")
public class LocationDriver extends GenericDriver<LocationProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult listRegions(String vendor) {
        LocationProvider provider = getProvider(vendor, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.listRegions();
    }

    public BocloudResult listRegions(String vendor, String regionId) {
        LocationProvider provider = getProvider(vendor, LocationProvider.class, repository, regionId);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.listRegions();
    }

    public BocloudResult listZones(String vendor, String region) {
        LocationProvider provider = getProvider(vendor, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.listZones(region);
    }

    public BocloudResult sync(String vendor) {
        LocationProvider provider = getProvider(vendor, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.sync();
    }

    public BocloudResult sync(String vendor, String username, String password, String projectId, String domainName) {
        LocationProvider provider = getProvider(vendor, username, password, projectId, domainName,
                LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.sync();
    }

    public BocloudResult syncStack(String vendor, LbListenerModel lbListenerModel) {
        LocationProvider provider = getProvider(vendor, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.syncStack(lbListenerModel);
    }

    public BocloudResult sync(String vendor, String regionId) {
        LocationProvider provider = getProvider(vendor, LocationProvider.class, repository, regionId);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.sync();
    }

    public BocloudResult syncPortByNetworkUuid(String vendor, String networkUuid) {
        LocationProvider provider = getProvider(vendor, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.syncPortByNetworkUuid(networkUuid);
    }

    public BocloudResult getHostModel(String vendor, HostModel host) {
        LocationProvider provider = getProvider(vendor, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.getHostModel(host);
    }

    public BocloudResult getManagedSystemSyncInfo(String vendorUuid, String uuid) {
        LocationProvider provider = getProvider(vendorUuid, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.getManagedSystemSyncInfo(uuid);
    }

    public BocloudResult<List<DataStoreModel>> syncStoreImages(String vendorUuid, String vdcName) {
        LocationProvider provider = getProvider(vendorUuid, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "云平台认证失败！");
        }
        return provider.syncStoreImages(vdcName);
    }

    public BocloudResult syncAwsImages(String vendorUuid, String regionId) {
        LocationProvider provider = getProvider(vendorUuid, LocationProvider.class, repository, regionId);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.syncAwsImages();
    }

    public BocloudResult syncAwsCost(String vendorUuid) {
        LocationProvider provider = getProvider(vendorUuid, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.syncAwsCost();
    }

    public BocloudResult syncAwsLoadBalancer(String vendorUuid, String regionId) {
        LocationProvider provider = getProvider(vendorUuid, LocationProvider.class, repository, regionId);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.syncAwsLoadBalancer();
    }

    public BocloudResult syncHostAggregate(String vendorUuid) {
        LocationProvider provider = getProvider(vendorUuid, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.syncHostAggregate();
    }

    public BocloudResult syncSecurityGroups(String vendorUuid) {
        LocationProvider provider = getProvider(vendorUuid, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.syncSecurityGroups();
    }

    public BocloudResult<List<LibraryModel>> listLibraries(String vendorUuid) {
        LocationProvider provider = getProvider(vendorUuid, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.listLibraries();
    }

    public BocloudResult syncAliCost(String vendorUuid, String billingCycle, String granularity, String billingDate) {
        LocationProvider provider = getProvider(vendorUuid, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.syncAliCost(billingCycle, granularity, billingDate);
    }

    public BocloudResult syncImages(String vendorUuid, String regionId) {
        LocationProvider provider = getProvider(vendorUuid, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.syncImages();
    }

    public BocloudResult summary(String vendor) {
        LocationProvider provider = getProvider(vendor, LocationProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.summary();
    }
}