package com.bocloud.cmp.driver.compute;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.model.ClusterOperateModel;
import com.bocloud.cmp.provider.compute.ClusterProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 集群操作服务接口实现类
 *
 * @author weiwei
 */
@Service("clusterDriver")
public class ClusterDriver extends GenericDriver<ClusterProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult create(String vendor, ClusterOperateModel model) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.create(model);
    }

    public BocloudResult remove(String vendor, ClusterOperateModel model) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.remove(model);
    }

    public BocloudResult detail(String vendor, String clusterVal) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.detail(clusterVal);
    }

    public BocloudResult listByVdc(String vendor, String vdcName) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.listByVdc(vdcName);
    }

    public BocloudResult detailCluster(String vendor, String clusterVal) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.detailCluster(clusterVal);
    }

    public BocloudResult create(String vendor, ClusterModel clusterModel) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.create(clusterModel);
    }

    public BocloudResult modify(String vendor, ClusterModel clusterModel) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.modify(clusterModel);
    }

    public BocloudResult remove(String vendor, String aggregateId) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.remove(aggregateId);
    }

    public BocloudResult addHost(String vendor, String aggregateId, String hostName) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.addHost(aggregateId, hostName);
    }

    public BocloudResult removeHost(String vendor, String aggregateId, String hostName) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.removeHost(aggregateId, hostName);
    }

    public BocloudResult summary(String vendor, String clusterVal) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.summary(clusterVal);
    }

    public BocloudResult queryByName(String vendor, String name) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.queryByName(name);
    }

    public BocloudResult resourceStatistics(String vendor, String aggregateId) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.resourceStatistics(aggregateId);
    }


    public BocloudResult<List<ClusterModel>> getClusterByVlanId(String vlanId, String vendor) {
        ClusterProvider clusterProvider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(clusterProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return clusterProvider.getClusterByVlanId(vlanId);
    }
}
