package com.bocloud.cmp.driver.compute;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.HmcHostModel;
import com.bocloud.cmp.provider.compute.HmcHostProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

/**
 * Hmc宿主机操作服务接口实现类
 *
 * @author lkx007
 */
@Repository("hmcHostDriver")
public class HmcHostDriver extends GenericDriver<HmcHostProvider> {


    @Autowired
    private ProviderRepository repository;

    public BocloudResult get(String vendor) {
        HmcHostProvider hmcHostProvider = getProvider(vendor, HmcHostProvider.class, repository);
        if (before(hmcHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return hmcHostProvider.list();
    }

    public BocloudResult get(String vendor, String hmcUuid) {
        HmcHostProvider hmcHostProvider = getProvider(vendor, HmcHostProvider.class, repository);
        if (before(hmcHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return hmcHostProvider.get(hmcUuid);
    }

    public BocloudResult create(String vendor, HmcHostModel hmc) {
        HmcHostProvider hmcHostProvider = getProvider(vendor, HmcHostProvider.class, repository);
        if (before(hmcHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return hmcHostProvider.create(hmc);
    }

    public BocloudResult modify(String vendor, HmcHostModel hmc) {
        HmcHostProvider hmcHostProvider = getProvider(vendor, HmcHostProvider.class, repository);
        if (before(hmcHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return hmcHostProvider.modify(hmc);
    }

    public BocloudResult delete(String vendor, String hmcUuid) {
        HmcHostProvider hmcHostProvider = getProvider(vendor, HmcHostProvider.class, repository);
        if (before(hmcHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return hmcHostProvider.delete(hmcUuid);
    }

    public BocloudResult getHosts(String vendor, String hmcUuid) {
        HmcHostProvider hmcHostProvider = getProvider(vendor, HmcHostProvider.class, repository);
        if (before(hmcHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return hmcHostProvider.getHosts(hmcUuid);
    }
}
