package com.bocloud.cmp.driver.compute;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.provider.compute.ClusterProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 主机聚合实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年7月5日
 */
@Service("hostAggregateDriver")
public class HostAggregateDriver extends GenericDriver<ClusterProvider> {
    @Autowired
    private ProviderRepository repository;

    public BocloudResult create(ClusterModel clusterModel, String vendor, String region) {
        ClusterProvider provider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(clusterModel);
    }


    public BocloudResult modify(ClusterModel clusterModel, String vendor, String region) {
        ClusterProvider provider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.modify(clusterModel);
    }


    public BocloudResult remove(String aggregateId, String vendor, String region) {
        ClusterProvider provider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(aggregateId);
    }


    public BocloudResult addHost(String aggregateId, String hostName, String vendor, String region) {
        ClusterProvider provider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.addHost(aggregateId, hostName);
    }


    public BocloudResult removeHost(String aggregateId, String hostName, String vendor, String region) {
        ClusterProvider provider = getProvider(vendor, ClusterProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.removeHost(aggregateId, hostName);
    }
}
