package com.bocloud.cmp.driver.compute;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.HostOperateModel;
import com.bocloud.cmp.provider.compute.HostProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

/**
 * 宿主机操作服务接口实现类
 *
 * @author weiwei
 */
@Repository("hostDriver")
public class HostDriver extends GenericDriver<HostProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult create(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.create(model);
    }

    public BocloudResult modify(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.modify(model);
    }

    public BocloudResult remove(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.remove(model);
    }

    public BocloudResult enterMode(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.enterMode(model);
    }

    public BocloudResult exitMode(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.exitMode(model);
    }

    public BocloudResult disconn(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.disconn(model);
    }

    public BocloudResult reconn(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.reconn(model);
    }

    public BocloudResult moveTo(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.moveTo(model);
    }

    public BocloudResult moveTo(String vendor, HostModel model,String clusterUrn) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.moveTo(model,clusterUrn);
    }

    public BocloudResult shutdown(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.shutdown(model);
    }

    public BocloudResult reboot(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.reboot(model);
    }

    public BocloudResult detail(String vendor, String region, HostOperateModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.detail(model);
    }

    public BocloudResult list(String vendor, String region, String clusterId) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.list(clusterId);
    }

    public BocloudResult list(String vendor, String region) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.list();
    }

    public BocloudResult listUnmanagedPowervm(String vendor, String hostname) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.listUnmanagedPowervm(hostname);
    }

    public BocloudResult listHosts(String vendor, String region) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.listHosts();
    }

    public BocloudResult listByCluster(String vendor, String clusterName) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.listByCluster(clusterName);
    }

    public BocloudResult create(String vendor, HostModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.create(model);
    }

    public BocloudResult modify(String vendor,  HostModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.modify(model);
    }

    public BocloudResult remove(String vendor, HostModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.remove(model);
    }

    public BocloudResult maintain(String vendor, HostModel model) {
        HostProvider hostProvider = getProvider(vendor, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.maintain(model);
    }


    public BocloudResult ifacesMgmt(String uuid) {
        HostProvider hostProvider = getProvider(uuid, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.ifacesMgmt();
    }

    public BocloudResult ifacesVxlan(String uuid) {
        HostProvider hostProvider = getProvider(uuid, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.ifacesVxlan();
    }

    public BocloudResult ifacesBusiness(String uuid) {
        HostProvider hostProvider = getProvider(uuid, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.ifacesBusiness();
    }

    public BocloudResult ifacesVs(String uuid) {
        HostProvider hostProvider = getProvider(uuid, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.ifacesVs();
    }

    public BocloudResult ifacesQos(String uuid) {
        HostProvider hostProvider = getProvider(uuid, HostProvider.class, repository);
        if (before(hostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return hostProvider.ifacesQos();
    }
}
