package com.bocloud.cmp.driver.compute;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.compute.ImageProvider;
import com.bocloud.common.model.BocloudResult;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author dmw
 */
@Service("imageDriver")
public class ImageDriver extends GenericDriver<ImageProvider> {
	@Autowired
	private ProviderRepository repository;

	public BocloudResult create(String vendor, String region, ImageModel imageModel) {
		ImageProvider imageProvider = getProvider(vendor, ImageProvider.class, repository, region);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.create(imageModel);
	}

	public BocloudResult create(String vendor, String projectName, String region, ImageModel imageModel) {
		ImageProvider imageProvider = getProvider(vendor, projectName, ImageProvider.class, repository, region);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.create(imageModel);
	}

	public BocloudResult list(String vendor, String region) {
		ImageProvider imageProvider = getProvider(vendor, ImageProvider.class, repository);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.list();
	}

	public BocloudResult detail(String imageId, String vendor, String region) {
		ImageProvider imageProvider = getProvider(vendor, ImageProvider.class, repository);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.detail(imageId);
	}

	public BocloudResult remove(String imageId, String vendor, String region, String vmName) {
		ImageProvider imageProvider = getProvider(vendor, ImageProvider.class, repository, region);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.remove(imageId);
	}

	public BocloudResult modify(String vendor, String region, ImageModel image) {
		ImageProvider imageProvider = getProvider(vendor, ImageProvider.class, repository, region);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.modify(image);
	}

	public BocloudResult shareImage(String vendor, String region, ImageModel image) {
		ImageProvider imageProvider = getProvider(vendor, ImageProvider.class, repository, region);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.shareImage(image);
	}

	public BocloudResult syncImages(String vendor, String region, ImageModel image) {
		ImageProvider imageProvider = getProvider(vendor, ImageProvider.class, repository, region);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.syncImages(image);
	}

	public BocloudResult download(String vendor, String projectName, String region, String imageId, String path) {
		ImageProvider imageProvider = getProvider(vendor, projectName, ImageProvider.class, repository, region);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.download(imageId, path);
	}

	public BocloudResult create(String vendor, String username, String password, String projectId, String domainName,
			ImageModel model) {
		ImageProvider imageProvider = getProvider(vendor, username, password, projectId, domainName,
				ImageProvider.class, repository);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.create(model);
	}

	public BocloudResult copy(ImageModel image, String vendor, String region) {
		ImageProvider imageProvider = getProvider(vendor, ImageProvider.class, repository, region);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.copy(image);
	}

	public BocloudResult list(String vendor, String region, String flavorId) {
		ImageProvider imageProvider = getProvider(vendor, ImageProvider.class, repository, region);
		if (before(imageProvider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return imageProvider.listByFlavor(flavorId);
	}
}
