package com.bocloud.cmp.driver.compute;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.OsHost;
import com.bocloud.cmp.provider.compute.OsHostProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 计算主机主机操作服务接口实现类
 *
 * @author lkx007
 */
@Repository("osHostDriver")
public class OsHostDriver extends GenericDriver<OsHostProvider> {
    @Autowired
    private ProviderRepository repository;

    public BocloudResult get(String vendor) {
        OsHostProvider osHostProvider = getProvider(vendor, OsHostProvider.class, repository);
        if (before(osHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return osHostProvider.get();
    }

    public BocloudResult get(String vendor, String hostName) {
        OsHostProvider osHostProvider = getProvider(vendor, OsHostProvider.class, repository);
        if (before(osHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return osHostProvider.get(hostName);
    }

    public BocloudResult create(String vendor, OsHost osHost) {
        OsHostProvider osHostProvider = getProvider(vendor, OsHostProvider.class, repository);
        if (before(osHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return osHostProvider.create(osHost);

    }

    public BocloudResult delete(String vendor, String hostName) {
        OsHostProvider osHostProvider = getProvider(vendor, OsHostProvider.class, repository);
        if (before(osHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return osHostProvider.delete(hostName);
    }

    /**
     * 管理小机上现有云主机
     *
     * @param hostName
     * @param serverUuids
     * @return
     */
    public BocloudResult onboard(String vendor, String hostName, List<String> serverUuids) {
        OsHostProvider osHostProvider = getProvider(vendor, OsHostProvider.class, repository);
        if (before(osHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return osHostProvider.onboard(hostName, serverUuids);
    }

    /**
     * 取消管理云主机
     *
     * @param hostName
     * @param serverUuids
     * @return
     */
    public BocloudResult unmanage(String vendor, String hostName, List<String> serverUuids) {
        OsHostProvider osHostProvider = getProvider(vendor, OsHostProvider.class, repository);
        if (before(osHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return osHostProvider.unmanage(hostName, serverUuids);
    }

    /**
     * 查询小机上所有云主机
     *
     * @param hostName
     * @return
     */
    public BocloudResult allServers(String vendor, String hostName) {
        OsHostProvider osHostProvider = getProvider(vendor, OsHostProvider.class, repository);
        if (before(osHostProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return osHostProvider.allServers(hostName);
    }
}
