package com.bocloud.cmp.driver.compute;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.fusionsphere.CdromModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.model.sangfor.SangforServerModel;
import com.bocloud.cmp.provider.compute.ServerProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 云主机操作服务接口实现类
 *
 * @author weiwei
 */
@Service("serverDriver")
public class ServerDriver extends GenericDriver<ServerProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.list();
    }

    public BocloudResult list(String vendor, String region, String hostId) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.list(hostId);
    }

    public BocloudResult getJnlpXml(String vendor, String region, Long vmId, String address) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.getJnlpXml(vmId, address);
    }

    public BocloudResult create(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.create(model);
    }

    public BocloudResult customizationVm(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.customizationVm(model);
    }

    public BocloudResult create(String vendor, String username, String password, String projectId, String domainName,
                                ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, username, password, projectId, domainName,
                ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.create(model);
    }

    public BocloudResult cloneAsVm(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.cloneAsVm(model);
    }

    public BocloudResult cloneAsTemplate(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.cloneAsTemplate(model);
    }

    public BocloudResult convertAsTemplate(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.convertAsTemplate(model);
    }

    public BocloudResult create(String vendor, String projectName, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, projectName, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.create(model);
    }

    public BocloudResult create(String vendor, String region, H3cServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.create(model);
    }

    public BocloudResult create(String vendor, String projectName, String region, H3cServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, projectName, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.create(model);
    }

    public BocloudResult migrate(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.migrate(model);
    }

    public BocloudResult migrate(String vendor, String region, OSMigrateModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.migrate(model);
    }

    public BocloudResult remove(String instanceId, String name, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.remove(instanceId, name);
    }

    public BocloudResult detail(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.detail(instanceId);
    }

    public BocloudResult enableDiskUuid(String instanceId, String vendor) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, null);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.enableDiskUuid(instanceId);
    }

    public BocloudResult deleteDisk(String vendor, String instanceId, List<String> diskUuids) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, null);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.deleteDisk(instanceId, diskUuids);
    }

    public BocloudResult selectDiskUuid(String vendor, String instanceId, List<String> diskUuids) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, null);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.selectDiskUuid(instanceId, diskUuids);
    }

    public BocloudResult detailFault(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.detailFault(instanceId);
    }

    public BocloudResult listByStack(HeatStackModel stackModel, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.listByStack(stackModel);
    }

    public BocloudResult start(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.start(instanceId);
    }

    public BocloudResult stop(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.stop(instanceId);
    }

    public BocloudResult softStop(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.softStop(instanceId);
    }

    public BocloudResult reboot(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.reboot(instanceId);
    }

    public BocloudResult softReboot(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.softReboot(instanceId);
    }

    public BocloudResult joinSGroup(String instanceId, String group, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.joinSGroup(instanceId, group);
    }

    public BocloudResult joinSGroup(String instanceId, String group, String vendor, String projectName, String region) {
        ServerProvider serverProvider = getProvider(vendor, projectName, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.joinSGroup(instanceId, group);
    }

    public BocloudResult allocationSGroups(String instanceId, List<String> groups, String vendor, String projectName,
                                           String region) {
        ServerProvider serverProvider = getProvider(vendor, projectName, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.allocationSGroups(instanceId, groups);
    }

    public BocloudResult allocationSGroups(String instanceId, List<String> groups, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.allocationSGroups(instanceId, groups);
    }

    public BocloudResult leaveSGroup(String instanceId, String group, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.leaveSGroup(instanceId, group);
    }

    public BocloudResult leaveSGroup(String instanceId, String group, String vendor, String projectName,
                                     String region) {
        ServerProvider serverProvider = getProvider(vendor, projectName, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.leaveSGroup(instanceId, group);
    }

    public BocloudResult modify(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.modify(model);
    }

    public BocloudResult modify(String vendor, String username, String password, String projectId, String domainName,
                                ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, username, password, projectId, domainName,
                ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.modify(model);
    }

    public BocloudResult snapshot(SnapshotModel snapshotModel, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.snapshot(snapshotModel);
    }

    public BocloudResult beforeCaptureImage(String vendor, String instanceId, ImageModel imageModel, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.beforeCaptureImage(instanceId, imageModel);
    }

    public BocloudResult captureImage(String instanceId, String imageName, List<String> volumes, String vendor,
                                      String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.createImage(instanceId, imageName, volumes);
    }

    public BocloudResult pause(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.pause(instanceId);
    }

    public BocloudResult unPause(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.unPause(instanceId);
    }

    public BocloudResult suspend(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.suspend(instanceId);
    }

    public BocloudResult resume(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.resume(instanceId);
    }

    public BocloudResult vncConsole(String vncType, String serverId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository,region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.vncConsole(vncType, serverId);
    }

    public BocloudResult vncConsole(String serverId, String vendor, String username, String password, String projectId,
                                    String domainName) {
        ServerProvider serverProvider = getProvider(vendor, username, password, projectId, domainName,
                ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.vncConsole(null, serverId);
    }

    public BocloudResult changePassword(String instanceId, String vendor, String region, String password) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.changePassword(instanceId, password);
    }

    public BocloudResult resize(String instanceId, String flavorId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.resize(instanceId, flavorId);
    }

    public BocloudResult resize(String instanceId, String flavorId, String vendor, String username, String password,
                                String projectId, String domainName) {
        ServerProvider serverProvider = getProvider(vendor, username, password, projectId, domainName,
                ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.resize(instanceId, flavorId);
    }

    public BocloudResult resize(ServerModel server, FlavorModel flavor, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.resize(server, flavor);
    }

    public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.resize(instanceId, flavorId, forceStop);
    }

    public BocloudResult confirmResize(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.confirmResize(instanceId);
    }

    public BocloudResult confirmResize(String instanceId, String vendor, String username, String password,
                                       String projectId, String domainName) {
        ServerProvider serverProvider = getProvider(vendor, username, password, projectId, domainName,
                ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.confirmResize(instanceId);
    }

    public BocloudResult revertResize(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.revertResize(instanceId);
    }

    public BocloudResult revertResize(String instanceId, String vendor, String username, String password,
                                      String projectId, String domainName) {
        ServerProvider serverProvider = getProvider(vendor, username, password, projectId, domainName,
                ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.revertResize(instanceId);
    }

    public BocloudResult attachNetwork(String vendor, String region, ServerModel server) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.attachNetwork(server);
    }

    public BocloudResult attachNetwork(String vendor, String projectName, String region, ServerModel server) {
        ServerProvider serverProvider = getProvider(vendor, projectName, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.attachNetwork(server);
    }

    public BocloudResult detachNetwork(String vendor, String region, String instanceId, String portUuid) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.detachNetwork(instanceId, portUuid);
    }

    public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable, String vendor,
                                         String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.hotConfigEnable(instanceId, cpuHotEnable, MemHotEnable);
    }

    public BocloudResult getTicket(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.getTicket(instanceId);
    }

    public BocloudResult createInResPool(String vendor, String region, ServerModel server) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.createInResPool(server);
    }

    public BocloudResult mountToolsInstaller(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.mountToolsInstaller(instanceId);
    }

    public BocloudResult unmountToolsInstaller(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.unmountToolsInstaller(instanceId);
    }

    public BocloudResult upgradeToolsTask(String instanceId, String options, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.upgradeToolsTask(instanceId, options);
    }

    public BocloudResult getNetCard(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.getNetCard(instanceId);
    }

    public BocloudResult getVmIps(String instanceId, String vendor) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.getVmIps(instanceId);
    }

    public BocloudResult getVmIps(String instanceId, String address, String vendor) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.getVmIps(instanceId, address);
    }

    public BocloudResult markAsTemplate(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.markAsTemplate(instanceId);
    }

    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd, String vendor) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, null);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.changeVspVmPasswd(instanceId, oldPasswd, newPasswd);
    }

    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd, String vendor,
                                           String regionId) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, regionId);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.changeVspVmPasswd(instanceId, oldPasswd, newPasswd);
    }

    public BocloudResult initVspVmPasswd(String instanceId, String oldPasswd, String newPasswd, String vendor) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, null);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.initVspVmPasswd(instanceId, oldPasswd, newPasswd);
    }

    public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize, String vendor,
                                    String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.diskExtend(instanceId, diskLabel, extendSize);
    }

    public BocloudResult getMetric(String type, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.getMetric(type);
    }

    public BocloudResult vmrcConsole(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.vmrcConsole(instanceId);
    }

    public BocloudResult vmToolsConfig(ServerModel serverModel, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.vmToolsConfig(serverModel);
    }

    public BocloudResult resetInstance(ServerModel serverModel, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.resetInstance(serverModel);
    }

    public BocloudResult bindKeypair(ServerModel serverModel, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.bindKeypair(serverModel);
    }

    public BocloudResult unbindKeypair(ServerModel serverModel, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.unbindKeypair(serverModel);
    }

    public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut, String vendor,
                                                      String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.resetInternetMaxBandwidthOut(instanceId, internetMaxBandwidthOut);
    }

    public BocloudResult attachVolume(String vendor, String region, String instanceId, VolumeModel volumeModel) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.attachVolume(instanceId, volumeModel);
    }

    public BocloudResult retachVolume(String vendor, String region, String instanceId, VolumeModel volumeModel) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.retachVolume(instanceId, volumeModel);
    }

    public BocloudResult detachVolume(String vendor, String region, String instanceId, VolumeModel volumeModel) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.detachVolume(instanceId, volumeModel);
    }

    public BocloudResult resetStatus(String vendor, String region, String instanceId) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.resetStatus(instanceId);
    }

    public BocloudResult deallocate(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.deallocate(instanceId);
    }

    public BocloudResult addPortPolicy(String vendor, String region, String portUuid, String policyId) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.addPortPolicy(policyId, portUuid);
    }

    public BocloudResult secondBindNics(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.secondBindNics(model);
    }

    public BocloudResult secondUnBindNics(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.secondUnBindNics(model);
    }

    public BocloudResult osDiskModify(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.osDiskModify(model);
    }

    public BocloudResult serverDetail(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.serverDetail(model);
    }

    public BocloudResult getVmFolder(String vendor, String vmFolder, String vdcName) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.getVmFolder(vmFolder, vdcName);
    }

    public BocloudResult getVmFolder(String vendor, String parentFolder, String vmFolder, String vdcName) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.getVmFolder(parentFolder, vmFolder, vdcName);
    }

    public BocloudResult rebuildServer(String vendor, SnapshotModel snapshotModel) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.rebuildServer(snapshotModel);
    }

    public BocloudResult joinSGroup(String vendor, String username, String password, String projectId,
                                    String domainName, String instanceId, String group) {
        ServerProvider serverProvider = getProvider(vendor, username, password, projectId, domainName,
                ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.joinSGroup(instanceId, group);
    }

    public BocloudResult leaveSGroup(String vendor, String username, String password, String projectId,
                                     String domainName, String instanceId, String secuName) {
        ServerProvider serverProvider = getProvider(vendor, username, password, projectId, domainName,
                ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.leaveSGroup(instanceId, secuName);
    }

    public BocloudResult attachNetwork(String vendor, String username, String password, String projectId,
                                       String domainName, ServerModel serverModel) {
        ServerProvider serverProvider = getProvider(vendor, username, password, projectId, domainName,
                ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.attachNetwork(serverModel);
    }

    public BocloudResult assignIpv6(String vendor, String networkInterfaceId, String ipv6, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.assignIpv6(networkInterfaceId, ipv6);
    }

    public BocloudResult setFields(String instanceId, String vendor, String region, List<TagModel> tagModels) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.setFields(instanceId, tagModels);
    }

    public BocloudResult createTag(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.createTag(model);
    }

    public BocloudResult removeTag(String vendor, String region, ServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.removeTag(model);
    }

    public BocloudResult removeFields(String instanceId, String vendor, String region, List<String> keys) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.removeFields(instanceId, keys);
    }

    public BocloudResult disableApiTermination(String vendor, String region, String instanceId, Boolean enabled) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.disableApiTermination(instanceId, enabled);
    }

    public BocloudResult listByHost(String vendor, String hostName) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, null);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.listByHost(hostName);
    }

    public BocloudResult remove(String instanceId, Boolean deletePublicIp, Boolean deleteVolume, String vendor,
                                String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.remove(instanceId, deletePublicIp, deleteVolume);
    }

    public BocloudResult availableSizes(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.availableSizes(instanceId);
    }

    public BocloudResult generalize(String vendor, String region, String instanceId) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.generalize(instanceId);
    }

    public BocloudResult hibernated(String instanceId, String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.hibernated(instanceId);
    }

    public BocloudResult attachCdrom(String vendor, CdromModel cdromModel) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.attachCdrom(cdromModel);
    }

    public BocloudResult detachCdrom(String vendor, CdromModel cdromModel) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.detachCdrom(cdromModel);
    }

    /**
     * 深信服克隆虚拟机
     * @param vendor
     * @param region
     * @param model
     * @return
     */
    public BocloudResult cloneSangfor(String vendor, String region, SangforServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.cloneVm(model);
    }

    /**
     * 深信服创建全新虚拟机
     * @param vendor
     * @param region
     * @param model
     * @return
     */
    public BocloudResult createSangfor(String vendor, String region, SangforServerModel model) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败!");
        }
        return serverProvider.createVm(model);
    }

    public BocloudResult listGroup(String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return BocloudResult.FAILED("云平台认证失败");
        }
        return serverProvider.listGroup();
    }

    public BocloudResult listLabel(String vendor, String region) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return BocloudResult.FAILED("云平台认证失败");
        }
        return serverProvider.listLabel();
    }
    
    public BocloudResult listCon(String vendor, String region, String storage) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return BocloudResult.FAILED("云平台认证失败");
        }
        return serverProvider.listCon(storage);
    }

    public BocloudResult listIso(String vendor, String region, String storage) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return BocloudResult.FAILED("云平台认证失败");
        }
        return serverProvider.listIso(storage);
    }

    public BocloudResult vnc(String vendor, String region, String instanceId) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return BocloudResult.FAILED("云平台认证失败");
        }
        return serverProvider.vnc(instanceId);
    }

    public BocloudResult vncLogin(String vendor, String region, String instanceId) {
        ServerProvider serverProvider = getProvider(vendor, ServerProvider.class, repository, region);
        if (before(serverProvider).isFailed()) {
            return BocloudResult.FAILED("云平台认证失败");
        }
        return serverProvider.vncLogin(instanceId);
    }
}
