package com.bocloud.cmp.driver.compute;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.ServerTemplateModel;
import com.bocloud.cmp.provider.compute.ServerTemplateProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

/**
 * 阿里云 云主机模板
 * 
 * @author ymh
 * @时间 2019年3月8日
 */
@Repository("serverTemplateDriver")
public class ServerTemplateDriver extends GenericDriver<ServerTemplateProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String region, ServerTemplateModel serverTemplateModel) {
		ServerTemplateProvider templateProvider = getProvider(vendor, ServerTemplateProvider.class, repository, region);
		if (before(templateProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return templateProvider.list(serverTemplateModel);
	}

	public BocloudResult detail(String vendor, String region, String uuid) {
		ServerTemplateProvider templateProvider = getProvider(vendor, ServerTemplateProvider.class, repository, region);
		if (before(templateProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return templateProvider.detail(uuid);
	}

	public BocloudResult create(String vendor, String region, ServerTemplateModel serverTemplateModel) {
		ServerTemplateProvider templateProvider = getProvider(vendor, ServerTemplateProvider.class, repository, region);
		if (before(templateProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return templateProvider.create(serverTemplateModel);
	}

	public BocloudResult remove(String vendor, String region, ServerTemplateModel serverTemplateModel) {
		ServerTemplateProvider templateProvider = getProvider(vendor, ServerTemplateProvider.class, repository, region);
		if (before(templateProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return templateProvider.remove(serverTemplateModel);
	}

	public BocloudResult modify(String vendor, String region, ServerTemplateModel serverTemplateModel) {
		ServerTemplateProvider templateProvider = getProvider(vendor, ServerTemplateProvider.class, repository, region);
		if (before(templateProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return templateProvider.modify(serverTemplateModel);
	}

	public BocloudResult createTemplateVersion(String vendor, String region, ServerTemplateModel serverTemplateModel) {
		ServerTemplateProvider templateProvider = getProvider(vendor, ServerTemplateProvider.class, repository, region);
		if (before(templateProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return templateProvider.createTemplateVersion(serverTemplateModel);
	}

	public BocloudResult removeTemplateVersion(String vendor, String region, ServerTemplateModel serverTemplateModel) {
		ServerTemplateProvider templateProvider = getProvider(vendor, ServerTemplateProvider.class, repository, region);
		if (before(templateProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return templateProvider.removeTemplateVersion(serverTemplateModel);
	}

	public BocloudResult modifyDefaultVersion(String vendor, String region, ServerTemplateModel serverTemplateModel) {
		ServerTemplateProvider templateProvider = getProvider(vendor, ServerTemplateProvider.class, repository, region);
		if (before(templateProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return templateProvider.modifyDefaultVersion(serverTemplateModel);
	}

	public BocloudResult createServer(String vendor, String region, String uuid, ServerModel serverModel) {
		ServerTemplateProvider templateProvider = getProvider(vendor, ServerTemplateProvider.class, repository, region);
		if (before(templateProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return templateProvider.createServer(uuid, serverModel);
	}

	public BocloudResult asServer(String vendor, String region, String uuid, ServerModel serverModel) {
		ServerTemplateProvider templateProvider = getProvider(vendor, ServerTemplateProvider.class, repository, region);
		if (before(templateProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return templateProvider.asServer(uuid, serverModel);
	}

}
