package com.bocloud.cmp.driver.compute;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.compute.SnapshotProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 快照操作服务接口实现类
 *
 * @author weiwei
 */
@Service("snapshotDriver")
public class SnapshotDriver extends GenericDriver<SnapshotProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String region) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.list();
	}

	public BocloudResult detail(String vendor, String region, SnapshotModel model) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.detail(model);
	}

	public BocloudResult create(String vendor, String region, SnapshotModel model) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.create(model);
	}

	public BocloudResult create(String vendor, String username, String password, String projectId, String domainName,
			SnapshotModel model) {
		SnapshotProvider provider = getProvider(vendor, username, password, projectId, domainName,
				SnapshotProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vendor, String region, SnapshotModel model) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.remove(model);
	}

	public BocloudResult revert(String vendor, String region, SnapshotModel model) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.revert(model);
	}

	public BocloudResult rebuild(String vendor, String region, SnapshotModel model) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.rebuild(model);
	}

	public BocloudResult removeByVolumeId(String volumeId, String vendor, String region) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.removeByVolumeId(volumeId);
	}

	public BocloudResult modify(SnapshotModel model, String vendor, String region) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.modify(model);
	}

	public BocloudResult getSnapshot(String snapshotId, String vendor, String region) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.getSnapshot(snapshotId);
	}

	public BocloudResult getAvaliableSnap(String snapshotId, String vendor, String region) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.getAvaliableSnap(snapshotId);
	}

	public BocloudResult detail(String vendor, String value, String serverName) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository, null);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.detail(value, serverName);
	}

	public BocloudResult removeServerSnapshot(SnapshotModel model, String vendor) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository, null);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.removeServerSnapshot(model);
	}

	public BocloudResult list(String vendor, String region,String serverInstanceId) {
		SnapshotProvider provider = getProvider(vendor, SnapshotProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败!");
		}
		return provider.list(serverInstanceId);
	}
}
