package com.bocloud.cmp.driver.database;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.database.DbInstanceDbModel;
import com.bocloud.cmp.model.database.DbInstanceModel;
import com.bocloud.cmp.model.database.DbInstanceUserModel;
import com.bocloud.cmp.provider.database.DatabaseProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("databaseDriver")
public class DatabaseDriver extends GenericDriver<DatabaseProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult create(String vendor, String region, DbInstanceModel database) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createInstance(database);
	}

	public BocloudResult detail(String instanceId, String vendor, String region) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.instanceDetail(instanceId);
	}

	public BocloudResult remove(String instanceId, String vendor, String region) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeInstance(instanceId);
	}

	public BocloudResult list(String vendor, String region) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.listInstances();
	}

	public BocloudResult reboot(String instanceId, String vendor, String region) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.rebootInstance(instanceId);
	}

	public BocloudResult getErrLog(DbInstanceModel dbInstanceModel, String vendor, String region) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.getErrLog(dbInstanceModel);
	}

	public BocloudResult getSlowLog(DbInstanceModel dbInstanceModel, String vendor, String region) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.getErrLog(dbInstanceModel);
	}

	public BocloudResult getRegions(String vendor, String region, String type, String volType) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.getRegions(type, volType);
	}

	public BocloudResult restartDBInstance(String vendor, String region, String instanceId) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.restartDBInstance(instanceId);
	}

	public BocloudResult createAccount(String vendor, String region, DbInstanceUserModel userModel) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createAccount(userModel);
	}

	public BocloudResult deleteAccount(String vendor, String region, String instanceId, String accountName) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.deleteAccount(instanceId, accountName);
	}

	public BocloudResult resetAccountPassword(String vendor, String region, String instanceId, String accountName,
			String accountPassword) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.resetAccountPassword(instanceId, accountName, accountPassword);
	}

	public BocloudResult resetAccount(String vendor, String region, String instanceId, String accountName,
			String accountPassword) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.resetAccount(instanceId, accountName, accountPassword);
	}

	public BocloudResult grantAccountPrivilege(String vendor, String region, String instanceId, String accountName,
			String dbName, String accountPrivilege) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.grantAccountPrivilege(instanceId, accountName, dbName, accountPrivilege);
	}

	public BocloudResult revokeAccountPrivilege(String vendor, String region, String instanceId, String accountName,
			String dbName) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.revokeAccountPrivilege(instanceId, accountName, dbName);
	}

	public BocloudResult createDatabase(String vendor, String region, DbInstanceDbModel dbModel) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createDatabase(dbModel);
	}

	public BocloudResult deleteDatabase(String vendor, String region, String instanceId, String dbName) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.deleteDatabase(instanceId, dbName);
	}

	public BocloudResult getAccount(String vendor, String region, String instanceId, String accountName) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.getAccount(instanceId, accountName);
	}

	public BocloudResult getDb(String vendor, String region, String instanceId, String dbName) {
		DatabaseProvider provider = getProvider(vendor, DatabaseProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.getDb(instanceId, dbName);
	}

}
