package com.bocloud.cmp.driver.f5;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.f5.NodeModel;
import com.bocloud.cmp.provider.f5.NodeProvider;
import com.bocloud.common.model.BocloudResult;

/**
 * f5 node 操作接口实现类
 * 
 * @author liuyuanyuan
 *
 */
@Service("f5NodeDriver")
public class F5NodeDriver extends GenericDriver<NodeProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		NodeProvider nodeProvider = getProvider(vendor, NodeProvider.class, repository);
		if (before(nodeProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return nodeProvider.list();
	}
	
	public BocloudResult stats(String vendor, String nodeName) {
		NodeProvider nodeProvider = getProvider(vendor, NodeProvider.class, repository);
		if (before(nodeProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return nodeProvider.stats(nodeName);
	}

	public BocloudResult create(String vendor, NodeModel nodeModel) {
		NodeProvider nodeProvider = getProvider(vendor, NodeProvider.class, repository);
		if (before(nodeProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return nodeProvider.create(nodeModel);
	}

	public BocloudResult modify(String vendor, NodeModel nodeModel) {
		NodeProvider nodeProvider = getProvider(vendor, NodeProvider.class, repository);
		if (before(nodeProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return nodeProvider.update(nodeModel);
	}

	public BocloudResult remove(String vendor, String name) {
		NodeProvider nodeProvider = getProvider(vendor, NodeProvider.class, repository);
		if (before(nodeProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return nodeProvider.remove(name);
	}
}
