package com.bocloud.cmp.driver.f5;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.f5.PoolMemberModel;
import com.bocloud.cmp.model.f5.PoolModel;
import com.bocloud.cmp.provider.f5.PoolProvider;
import com.bocloud.common.model.BocloudResult;

/**
 * f5 pool 操作接口实现类
 * 
 * @author liuyuanyuan
 *
 */
@Service("f5PoolDriver")
public class F5PoolDriver extends GenericDriver<PoolProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		PoolProvider poolProvider = getProvider(vendor, PoolProvider.class, repository);
		if (before(poolProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return poolProvider.list();
	}

	public BocloudResult create(String vendor, PoolModel poolModel) {
		PoolProvider poolProvider = getProvider(vendor, PoolProvider.class, repository);
		if (before(poolProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return poolProvider.create(poolModel);
	}

	public BocloudResult createPoolMember(String vendor, PoolMemberModel poolMemberModel, String url) {
		PoolProvider poolProvider = getProvider(vendor, PoolProvider.class, repository);
		if (before(poolProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return poolProvider.createPoolMember(poolMemberModel, url);
	}

	public BocloudResult modify(String vendor, PoolModel poolModel) {
		PoolProvider poolProvider = getProvider(vendor, PoolProvider.class, repository);
		if (before(poolProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return poolProvider.update(poolModel);
	}

	public BocloudResult remove(String vendor, String name) {
		PoolProvider poolProvider = getProvider(vendor, PoolProvider.class, repository);
		if (before(poolProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return poolProvider.remove(name);
	}
	
	public BocloudResult removePoolMember(String vendor, String url) {
		PoolProvider poolProvider = getProvider(vendor, PoolProvider.class, repository);
		if (before(poolProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return poolProvider.removePoolMember(url);
	}
	
	public BocloudResult updatePoolMember(String vendor, String url, PoolMemberModel poolMemberModel) {
		PoolProvider poolProvider = getProvider(vendor, PoolProvider.class, repository);
		if (before(poolProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return poolProvider.updatePoolMember(url, poolMemberModel);
	}

	public BocloudResult detail(String vendor, String url) {
		PoolProvider poolProvider = getProvider(vendor, PoolProvider.class, repository);
		if (before(poolProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return poolProvider.detail(url);
	}
	
	public BocloudResult stats(String vendor, String url) {
		PoolProvider poolProvider = getProvider(vendor, PoolProvider.class, repository);
		if (before(poolProvider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败!");
		}
		return poolProvider.stats(url);
	}
}
