package com.bocloud.cmp.driver.hmc;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroup;
import com.bocloud.cmp.provider.hmc.VolumeGroupProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 存储卷组Driver
 *
 * @author sxp
 * @since Nov 20, 2018
 */
@Service("hmcVolumeGroupDriver")
public class HmcVolumeGroupDriver extends GenericDriver<VolumeGroupProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult getVolumeGroup(String vendorUuid, String viosUuid) {
        VolumeGroupProvider provider = getProvider(vendorUuid, VolumeGroupProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.getVolumeGroup(viosUuid);
    }

    public BocloudResult getVolumeGroup(String vendorUuid, String viosUuid, String volumeGroupUuid) {
        VolumeGroupProvider provider = getProvider(vendorUuid, VolumeGroupProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.getVolumeGroup(viosUuid, volumeGroupUuid);
    }

    public BocloudResult createVirtualDisk(String vendorUuid, String viosUuid, String volumeGroupUuid,
                                           Double diskCapacity, String diskName) {
        VolumeGroupProvider provider = getProvider(vendorUuid, VolumeGroupProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.createVirtualDisk(viosUuid, volumeGroupUuid, diskCapacity, diskName);
    }

    public BocloudResult deleteVirtualDisk(String vendorUuid, String viosUuid, String volumeGroupUuid,
                                           String diskName) {
        VolumeGroupProvider provider = getProvider(vendorUuid, VolumeGroupProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.deleteVirtualDisk(viosUuid, volumeGroupUuid, diskName);
    }

    public BocloudResult assign(String vendorUuid, String managedSystemUuid, String viosUuid, String lparUuid,
                                String diskName, String diskCapacity, Integer viosId, Integer lparId, Integer lparSlot,
                                boolean isPhysical) {
        VolumeGroupProvider provider = getProvider(vendorUuid, VolumeGroupProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.assign(managedSystemUuid, viosUuid, lparUuid, diskName, diskCapacity, viosId, lparId, lparSlot,
                isPhysical);
    }

    public BocloudResult unAssign(String vendorUuid, String managedSystemUuid, String viosUuid, String diskName,
                                  boolean isPhysical) {
        VolumeGroupProvider provider = getProvider(vendorUuid, VolumeGroupProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.unAssign(managedSystemUuid, viosUuid, diskName, isPhysical);
    }

    public BocloudResult updateVirtualDisk(String vendorUuid, String viosUuid, String volumeGroupUuid,
                                           Double diskCapacity, String diskName) {
        VolumeGroupProvider provider = getProvider(vendorUuid, VolumeGroupProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.updateVirtualDisk(viosUuid, volumeGroupUuid, diskCapacity, diskName);
    }

    public BocloudResult createVolumeGroup(String vendorUuid, String viosUuid, VolumeGroup volumeGroup) {
        VolumeGroupProvider provider = getProvider(vendorUuid, VolumeGroupProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.createVolumeGroup(viosUuid, volumeGroup);
    }

    public BocloudResult deleteVolumeGroup(String vendorUuid, String viosUuid, String volumeGroupUuid) {
        VolumeGroupProvider provider = getProvider(vendorUuid, VolumeGroupProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.deleteVolumeGroup(viosUuid, volumeGroupUuid);
    }

}
