package com.bocloud.cmp.driver.jumpserver;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.jumpserver.CmdTemplate;
import com.bocloud.cmp.model.jumpserver.CmdTemplateRule;
import com.bocloud.cmp.provider.jumpserver.CmdTemplateProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @description:
 * @author: Li Yaohui
 * @since: 2021/5/11 4:21 下午
 */
@Service
public class CmdTemplateDriver extends GenericDriver<CmdTemplateProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, Integer page, Integer size) {
        CmdTemplateProvider provider = getProvider(vendor, CmdTemplateProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.list(page, size);
    }

    public BocloudResult<CmdTemplate> create(String vendor, CmdTemplate cmdTemplate) {
        CmdTemplateProvider provider = getProvider(vendor, CmdTemplateProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.create(cmdTemplate);
    }

    public BocloudResult modify(String vendor, CmdTemplate cmdTemplate) {
        CmdTemplateProvider provider = getProvider(vendor, CmdTemplateProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.modify(cmdTemplate);
    }

    public BocloudResult remove(String vendor, Long id) {
        CmdTemplateProvider provider = getProvider(vendor, CmdTemplateProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.remove(id);
    }

    public BocloudResult detail(String vendor, Long id) {
        CmdTemplateProvider provider = getProvider(vendor, CmdTemplateProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.detail(id);
    }

    public BocloudResult<List<CmdTemplateRule>> listRuleByTempId(String vendor, Long id) {
        CmdTemplateProvider provider = getProvider(vendor, CmdTemplateProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.listRuleByTempId(id);
    }

    public BocloudResult<List<CmdTemplateRule>> listCreateTempRule(String vendor, List<CmdTemplateRule> cmdTemplateRules) {
        CmdTemplateProvider provider = getProvider(vendor, CmdTemplateProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.listCreateTempRule(cmdTemplateRules);
    }

    public BocloudResult<Void> removeTempRuleByRuleId(String vendor, Long ruleId) {
        CmdTemplateProvider provider = getProvider(vendor, CmdTemplateProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.removeTempRuleByRuleId(ruleId);
    }
}
