package com.bocloud.cmp.driver.jumpserver;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.jumpserver.ResourceAudit;
import com.bocloud.cmp.provider.jumpserver.ResourceAuditProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service("jumpserverResourceAuditDriver")
public class ResourceAuditDriver extends GenericDriver<ResourceAuditProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult<List<ResourceAudit>> listByDate(String vendor, String date) {
        ResourceAuditProvider provider = getProvider(vendor, ResourceAuditProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "云平台认证失败！");
        }
        return provider.listByDate(date);
    }

    public BocloudResult killSession(String vendor, String id) {
        ResourceAuditProvider provider = getProvider(vendor, ResourceAuditProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.killSession(id);
    }

    public BocloudResult getById(String vendor, String id) {
        ResourceAuditProvider provider = getProvider(vendor, ResourceAuditProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        try {
            return provider.getById(id);
        } catch (Exception e) {
            return new BocloudResult(false, "切断会话失败！");
        }
    }

    public BocloudResult getTuiRoute(String id, String vendor) {
        ResourceAuditProvider provider = getProvider(vendor, ResourceAuditProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.getTuiRoute(id);
    }

    public BocloudResult getFileRoute(String sessid, String devPath, String vendor) {
        ResourceAuditProvider provider = getProvider(vendor, ResourceAuditProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.getFileRoute(sessid, devPath);
    }

    public BocloudResult downloadGui(String id, String vendor) {
        ResourceAuditProvider provider = getProvider(vendor, ResourceAuditProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.downloadGui(id);
    }

    public BocloudResult downloadTui(String route, String vendor) {
        ResourceAuditProvider provider = getProvider(vendor, ResourceAuditProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.downloadTui(route);
    }

    public BocloudResult sessionPlayback(String vendor, String type, String sessId, Integer location) {
        ResourceAuditProvider provider = getProvider(vendor, ResourceAuditProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.sessionPlayback(type, sessId, location);
    }
}
