package com.bocloud.cmp.driver.loadbalance;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.loadbalance.LoadBalanceProvider;
import com.bocloud.common.model.BocloudResult;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("lbDriver")
public class LbDriver extends GenericDriver<LoadBalanceProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String region) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, String region, LbModel lb) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(lb);
	}

	public BocloudResult create(String vendor, String projectName, String region, LbModel lb) {
		LoadBalanceProvider provider = getProvider(vendor, projectName, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(lb);
	}

	public BocloudResult update(String vendor, String region, LbModel lb) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(lb);
	}

	public BocloudResult remove(String lbId, String vendor, String region) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(lbId);
	}

	public BocloudResult detail(String lbId, String vendor, String region) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(lbId);
	}

	public BocloudResult listVip(String vendor, String region) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.listVip();
	}

	public BocloudResult createVip(String vendor, String region, VipModel vModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createVip(vModel);
	}

	public BocloudResult createVip(String vendor, String projectName, String region, VipModel vModel) {
		LoadBalanceProvider provider = getProvider(vendor, projectName, LoadBalanceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createVip(vModel);
	}

	public BocloudResult updateVip(String vendor, String region, VipModel vModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.updateVip(vModel);
	}

	public BocloudResult removeVip(String vendor, String region, String vipId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeVip(vipId);
	}

	public BocloudResult detailVip(String vendor, String region, String vipId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detailVip(vipId);
	}

	public BocloudResult listListener(String vendor, String region) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.listListener();
	}

	public BocloudResult listListener(String vendor, String region, String lbId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.listListener(lbId);
	}

	public BocloudResult createListener(String vendor, String region, LbListenerModel lbListenerModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createListener(lbListenerModel);
	}

	public BocloudResult createListener(String vendor, String projectName, String region,
			LbListenerModel lbListenerModel) {
		LoadBalanceProvider provider = getProvider(vendor, projectName, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createListener(lbListenerModel);
	}

	public BocloudResult updateListener(String vendor, String region, LbListenerModel lbListenerModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.updateListener(lbListenerModel);
	}

	public BocloudResult removeListener(String vendor, String region, String listenerId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeListener(listenerId);
	}

	public BocloudResult detailListener(String vendor, String region, String listenerId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detailListener(listenerId);
	}

	public BocloudResult listHealMonitor(String vendor, String region) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.listHealMonitor();
	}

	public BocloudResult createHealMonitor(String vendor, String region, HealthMonitorModel hmModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createHealMonitor(hmModel);
	}

	public BocloudResult createHealMonitor(String vendor, String projectName, String region,
			HealthMonitorModel hmModel) {
		LoadBalanceProvider provider = getProvider(vendor, projectName, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createHealMonitor(hmModel);
	}

	public BocloudResult updateHealMonitor(String vendor, String region, HealthMonitorModel hmModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.updateHealMonitor(hmModel);
	}

	public BocloudResult removeHealMonitor(String vendor, String region, String hmId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeHealMonitor(hmId);
	}

	public BocloudResult detailHealMonitor(String vendor, String region, String hmId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detailHealMonitor(hmId);
	}

	public BocloudResult listMember(String vendor, String region, String poolId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.listMember(poolId);
	}

	public BocloudResult createMember(String vendor, String region, LbMemberModel mbModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createMember(mbModel);
	}

	public BocloudResult createMember(String vendor, String projectName, String region, LbMemberModel mbModel) {
		LoadBalanceProvider provider = getProvider(vendor, projectName, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createMember(mbModel);
	}

	public BocloudResult updateMember(String vendor, String region, LbMemberModel mbModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.updateMember(mbModel);
	}

	public BocloudResult removeMember(String vendor, String region, String poolId, String mbId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeMember(poolId, mbId);
	}

	public BocloudResult detailMember(String vendor, String region, String poolId, String mbId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detailMember(poolId, mbId);
	}

	public BocloudResult listPool(String vendor, String region) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.listPool();
	}

	public BocloudResult createPool(String vendor, String region, LbPoolModel poolModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createPool(poolModel);
	}

	public BocloudResult createPool(String vendor, String region, LbPoolModel poolModel,
			List<LbMemberModel> memberModels) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createPool(poolModel, memberModels);
	}

	public BocloudResult createPool(String vendor, String projectName, String region, LbPoolModel poolModel) {
		LoadBalanceProvider provider = getProvider(vendor, projectName, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createPool(poolModel);
	}

	public BocloudResult removePool(String vendor, String region, String poolId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removePool(poolId);
	}

	public BocloudResult updatePool(String vendor, String region, LbPoolModel poolModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.updatePool(poolModel);
	}
	
	public BocloudResult updatePool(String vendor, String region, LbPoolModel poolModel, List<LbMemberModel> memberModels) {
        LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.updatePool(poolModel, memberModels);
    }

	public BocloudResult detailPool(String vendor, String region, String poolId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detailPool(poolId);
	}

	public BocloudResult update(String vendor, String region, LbModel lb, String action) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(lb, action);
	}

	public BocloudResult removeListener(String vendor, String region, String lbId, String protocol, Integer port) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeListener(port, lbId, protocol);
	}

	public BocloudResult addMembers(String vendor, String vServerGroupId, String region,
			List<LbMemberModel> memberModels) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.addMembers(vServerGroupId, memberModels);
	}

	public BocloudResult removeMembers(String vendor, String region, String vServerGroupId,
			List<LbMemberModel> memberModels) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeMembers(vServerGroupId, memberModels);
	}

	public BocloudResult createListenerPolicy(String vendor, String region, LbListenerPolicyModel policyModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createListenerPolicy(policyModel);
	}

	public BocloudResult removeListenerPolicy(String vendor, String region, String policyId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeListenerPolicy(policyId);
	}

	public BocloudResult updateListenerPolicy(String vendor, String region, LbListenerPolicyModel policyModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.updateListenerPolicy(policyModel);
	}

	public BocloudResult createListenerRule(String vendor, String region, LbListenerRuleModel ruleModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createListenerRule(ruleModel);
	}

	public BocloudResult removeListenerRule(String vendor, String region, String policyId, String ruleId) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeListenerRule(policyId, ruleId);
	}

	public BocloudResult updateListenerRule(String vendor, String region, LbListenerRuleModel ruleModel) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.updateListenerRule(ruleModel);
	}

	public BocloudResult listListenerRule(String vendor, String policyId, String region) {
		LoadBalanceProvider provider = getProvider(vendor, LoadBalanceProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.listListenerRule(policyId);
	}
}
