package com.bocloud.cmp.driver.monitor;

import java.util.List;

import com.bocloud.cmp.model.fusionsphere.FusionSphereMonitorModel;
import com.bocloud.cmp.model.fusionsphere.FusionSphereMonitorRealTimeModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.monitor.MonitorProvider;
import com.bocloud.common.model.BocloudResult;

/**
 * 性能数据服务接口实现类
 * 
 * @author weiwei
 *
 */
@Service("monitorDriver")
public class MonitorDriver extends GenericDriver<MonitorProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult query(String vendor, String name, String category, List<String> types, String time) {
        MonitorProvider provider = getProvider(vendor, MonitorProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.query(time, name, category, types);
    }
    public BocloudResult metricNamespaces(String vendor, String resourceUri) {
        MonitorProvider provider = getProvider(vendor, MonitorProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.metricNamespaces(resourceUri);
    }

    public BocloudResult metricDefinitions(String vendor, String resourceUri, String metricNamespace) {
        MonitorProvider provider = getProvider(vendor, MonitorProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.metricDefinitions(resourceUri, metricNamespace);
    }

    public BocloudResult metrics(String vendor, String resourceUri, String timespan, String interval
            , String metricnames, String aggregation, String metricNamespace) {
        MonitorProvider provider = getProvider(vendor, MonitorProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.metrics(resourceUri, timespan, interval, metricnames, aggregation, metricNamespace);
    }

    public BocloudResult metrics(String vendor, FusionSphereMonitorModel model , String siteUri ) {
        MonitorProvider provider = getProvider(vendor, MonitorProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.metrics(model,siteUri);
    }

    public BocloudResult metrics(String vendor, FusionSphereMonitorRealTimeModel model , String siteUri ) {
        MonitorProvider provider = getProvider(vendor, MonitorProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.realMetrics(model);
    }
}
