package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.ElasticIpModel;
import com.bocloud.cmp.provider.network.ElasticIpProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * AWS弹性IP实现类
 *
 * @author
 */
@Service("elasticIpDriver")
public class ElasticIpDriver extends GenericDriver<ElasticIpProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult release(String vendor, String allocationId, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.release(allocationId);
    }

    public BocloudResult allocate(String vendor, String name, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.allocate(name);

    }

    public BocloudResult associateNetworkInterface(String vendor, String allocationId, String networkInterfaceId,
                                                   String privateIpAddress, boolean allowReassociation, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.associateNetworkInterface(allocationId, networkInterfaceId, privateIpAddress,
                allowReassociation);
    }

    public BocloudResult associateInstance(String vendor, String allocationId, String instanceId,
                                           String privateIpAddress, boolean allowReassociation, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.associateInstance(allocationId, instanceId, privateIpAddress, allowReassociation);
    }

    public BocloudResult disassociate(String vendor, String associationId, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.disassociate(associationId);
    }

    public BocloudResult update(String vendor, ElasticIpModel model, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.update(model);
    }

    public BocloudResult allocate(ElasticIpModel model, String vendor, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.allocate(model);
    }

    public BocloudResult associate(ElasticIpModel model, String vendor, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.associate(model);
    }

    public BocloudResult disassociate(ElasticIpModel model, String vendor, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.disassociate(model);
    }

    public BocloudResult detail(String allocationId, String vendor, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.detail(allocationId);
    }

    public BocloudResult updateBandwidth(String vendor, ElasticIpModel model, String region) {
        ElasticIpProvider provider = getProvider(vendor, ElasticIpProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.updateBandwidth(model);
    }
}
