package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.FloatingIpModel;
import com.bocloud.cmp.provider.network.FloatingIPProvider;
import com.bocloud.common.model.BocloudResult;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 浮动IP provider实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年8月28日
 */
@Service("floatIPDriver")
public class FloatIPDriver extends GenericDriver<FloatingIPProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, FloatingIpModel floatingIp) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(floatingIp);
	}

	public BocloudResult create(String vendor, String username, String password, String projectId, String domainName,
			FloatingIpModel floatingIp) {
		FloatingIPProvider provider = getProvider(vendor, username, password, projectId, domainName,
				FloatingIPProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(floatingIp);
	}

	public BocloudResult create(String vendor, String projectName, FloatingIpModel floatingIp) {
		FloatingIPProvider provider = getProvider(vendor, projectName, FloatingIPProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(floatingIp);
	}

	public BocloudResult create(String vendor, FloatingIpModel floatingIp, String regionId) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(floatingIp);
	}

	public BocloudResult create(String vendor, String projectName, FloatingIpModel floatingIp, String regionId) {
		FloatingIPProvider provider = getProvider(vendor, projectName, FloatingIPProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(floatingIp);
	}

	public BocloudResult modify(String vendor, FloatingIpModel floatingIp, String regionId) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.modify(floatingIp);
	}

	public BocloudResult remove(String fipId, String vendor) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.remove(fipId);
	}

	public BocloudResult remove(String fipId, String vendor, String regionId) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.remove(fipId);
	}

	public BocloudResult detail(String fipId, String vendor) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.detail(fipId);
	}

	public BocloudResult associate(String fipId, String portId, String vendor) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.associate(fipId, portId);
	}

	public BocloudResult associate(String vendor, FloatingIpModel floatingIp, String regionId) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.associate(floatingIp);
	}

	public BocloudResult disassociate(String fipId, String vendor) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.disassociate(fipId);
	}

	public BocloudResult disassociate(String fipId, String vendor, String region) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.disassociate(fipId);
	}

	public BocloudResult disassociate(String vendor, FloatingIpModel floatingIp, String regionId) {
		FloatingIPProvider provider = getProvider(vendor, FloatingIPProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.disassociate(floatingIp);
	}

	public BocloudResult create(String vendor, String username, String password, String projectId, String domainName,
			FloatingIpModel model, String regionId) {
		FloatingIPProvider provider = getProvider(vendor, username, password, projectId, domainName,
				FloatingIPProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(model);
	}

}
