package com.bocloud.cmp.driver.network;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.GwQosModel;
import com.bocloud.cmp.provider.network.GwQosProvider;
import com.bocloud.common.model.BocloudResult;

/**
 * 电科云 qos
 * 
 * @author ymh
 * @时间 2019年11月12日
 */
@Service("gwQosDriver")
public class GwQosDriver extends GenericDriver<GwQosProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult create(String vendor, GwQosModel gwQosModel) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(gwQosModel);
    }

    public BocloudResult modify(String vendor, GwQosModel gwQosModel) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.modify(gwQosModel);
    }

    public BocloudResult remove(String gwQosId, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(gwQosId);
    }

    public BocloudResult detail(String gwQosId, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.detail(gwQosId);
    }

    public BocloudResult qosRulesList(GwQosModel gwQosModel, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosRulesList(gwQosModel);
    }

    public BocloudResult qosRulesCreate(GwQosModel gwQosModel, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosRulesCreate(gwQosModel);
    }

    public BocloudResult qosRulesCreate(GwQosModel gwQosModel, String projectName, String vendor) {
        GwQosProvider provider = getProvider(vendor, projectName, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosRulesCreate(gwQosModel);
    }

    public BocloudResult qosRulesDetail(GwQosModel gwQosModel, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosRulesDetail(gwQosModel);
    }

    public BocloudResult qosRulesModify(GwQosModel gwQosModel, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosRulesModify(gwQosModel);
    }

    public BocloudResult qosRulesRemove(GwQosModel gwQosModel, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosRulesRemove(gwQosModel);
    }

    public BocloudResult qosRulesRemove(GwQosModel gwQosModel, String projectName, String vendor) {
        GwQosProvider provider = getProvider(vendor, projectName, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosRulesRemove(gwQosModel);
    }

    public BocloudResult qosPoliciesList(GwQosModel gwQosModel, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosPoliciesList(gwQosModel);
    }

    public BocloudResult qosPolicyCreate(GwQosModel gwQosModel, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosPolicyCreate(gwQosModel);
    }

    public BocloudResult qosPolicyCreate(GwQosModel gwQosModel, String projectName, String vendor) {
        GwQosProvider provider = getProvider(vendor, projectName, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosPolicyCreate(gwQosModel);
    }

    public BocloudResult qosPolicyDetail(GwQosModel gwQosModel, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosPolicyDetail(gwQosModel);
    }

    public BocloudResult qosPolicyModify(GwQosModel gwQosModel, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosPolicyModify(gwQosModel);
    }

    public BocloudResult qosPolicyModify(GwQosModel gwQosModel, String projectName, String vendor) {
        GwQosProvider provider = getProvider(vendor, projectName, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosPolicyModify(gwQosModel);
    }

    public BocloudResult qosPolicyRemove(GwQosModel gwQosModel, String vendor) {
        GwQosProvider provider = getProvider(vendor, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosPolicyRemove(gwQosModel);
    }

    public BocloudResult qosPolicyRemove(GwQosModel gwQosModel, String projectName, String vendor) {
        GwQosProvider provider = getProvider(vendor, projectName, GwQosProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.qosPolicyRemove(gwQosModel);
    }
}
