package com.bocloud.cmp.driver.network;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.InternetGatewayModel;
import com.bocloud.cmp.provider.network.InternetGatewayProvider;
import com.bocloud.common.model.BocloudResult;

/**
 * AWS互联网网关实现类
 *
 * @author
 */
@Service("internetGatewayDriver")
public class InternetGatewayDriver extends GenericDriver<InternetGatewayProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String region) {
		InternetGatewayProvider provider = getProvider(vendor, InternetGatewayProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, String name, String region) {
		InternetGatewayProvider provider = getProvider(vendor, InternetGatewayProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(name);
	}

	public BocloudResult remove(String vendor, String internetGatewayId, String region) {
		InternetGatewayProvider provider = getProvider(vendor, InternetGatewayProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.remove(internetGatewayId);

	}

	public BocloudResult attach(String vendor, String internetGatewayId, String vpcId, String region) {
		InternetGatewayProvider provider = getProvider(vendor, InternetGatewayProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.attach(internetGatewayId, vpcId);
	}

	public BocloudResult detach(String vendor, String internetGatewayId, String vpcId, String region) {
		InternetGatewayProvider provider = getProvider(vendor, InternetGatewayProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.detach(internetGatewayId, vpcId);
	}

	public BocloudResult update(String vendor, InternetGatewayModel model, String region) {
		InternetGatewayProvider provider = getProvider(vendor, InternetGatewayProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.update(model);
	}

	public BocloudResult create(InternetGatewayModel model,String vendor, String region) {
		InternetGatewayProvider provider = getProvider(vendor, InternetGatewayProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(model);
	}
}
