package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.provider.network.NetworkCardProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 网络操作服务接口实现类
 *
 * @author
 */
@Service("networkCardDriver")
public class NetworkCardDriver extends GenericDriver<NetworkCardProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String region) {
		NetworkCardProvider provider = getProvider(vendor, NetworkCardProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult detail(String vendor, String networkCardId, String region) {
		NetworkCardProvider provider = getProvider(vendor, NetworkCardProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.detail(networkCardId);
	}

	public BocloudResult attachNetworkCard(String vendor, NetworkCardModel networkCardModel, String region) {
		NetworkCardProvider provider = getProvider(vendor, NetworkCardProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.attachNetworkCard(networkCardModel);
	}

	public BocloudResult detachNetworkCard(String vendor, NetworkCardModel networkCardModel, String region) {
		NetworkCardProvider provider = getProvider(vendor, NetworkCardProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.detachNetworkCard(networkCardModel);
	}

	public BocloudResult remove(String vendor, String networkCardId, String region) {
		NetworkCardProvider provider = getProvider(vendor, NetworkCardProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.remove(networkCardId);
	}

	public BocloudResult create(String vendor, NetworkCardModel networkCardModel, String region) {
		NetworkCardProvider provider = getProvider(vendor, NetworkCardProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(networkCardModel);
	}
}
