package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.model.PortGroupModel;
import com.bocloud.cmp.model.SwitchModel;
import com.bocloud.cmp.model.sangfor.SangforVlanGroupModel;
import com.bocloud.cmp.provider.network.NetworkProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 网络操作服务接口实现类
 *
 * @author weiwei
 */
@Service("networkDriver")
public class NetworkDriver extends GenericDriver<NetworkProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, String region) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult create(String vendor, String region, NetworkModel model) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult create(String vendor, String username, String password, String projectId, String domainName,
                                NetworkModel model) {
        NetworkProvider provider = getProvider(vendor, username, password, projectId, domainName, NetworkProvider.class,
                repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult create(String vendor, String projectName, String region, NetworkModel model) {
        NetworkProvider provider = getProvider(vendor, projectName, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult remove(String networkId, String vendor, String region) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(networkId);
    }

    public BocloudResult remove(String switchId, String networkId, String vendor, String region) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(switchId, networkId);
    }

    public BocloudResult detail(String networkId, String vendor, String region) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.detail(networkId);
    }

    public BocloudResult detail(String switcherUuid, String networkId, String vendor, String region) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.detail(switcherUuid, networkId);
    }

    public BocloudResult detail(String vendor) {
        NetworkProvider networkProvider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(networkProvider).isFailed()) {
            return new BocloudResult(false, "用户鉴权失败！");
        } else {
            return networkProvider.list();
        }
    }

    public BocloudResult modify(String vendor, String region, NetworkModel model) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult createDSwitch(String vendor, SwitchModel switchModel) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.createDSwitch(switchModel);
    }

    public BocloudResult createVSwitch(String vendor, SwitchModel switchModel) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.createVSwitch(switchModel);
    }

    public BocloudResult removeDSwitch(String vendor, String name) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.removeDSwitch(name);
    }

    public BocloudResult renameDSwitch(String vendor, String oldName, String newName) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.renameDSwitch(oldName, newName);
    }

    public BocloudResult removeVSwitch(String vendor, String hostName, String vswitchName, List<String> pgNames) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.removeVSwitch(hostName, vswitchName, pgNames);
    }

    public BocloudResult updateVSwitch(String vendor, String hostName, String vswitchName, SwitchModel model) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.updateVSwitch(hostName, vswitchName, model);
    }

    public BocloudResult updateDVSwitch(String vendor, SwitchModel model) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.updateDVSwitch(model);
    }

    public BocloudResult createPortGroup(String vendor, String hostName, String vswitchName, PortGroupModel pgModel) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.createPortGroup(hostName, vswitchName, pgModel);
    }

    public BocloudResult removePortGroup(String vendor, String hostName, String pgName, String type) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.removePortGroup(hostName, pgName, type);
    }

    public BocloudResult updatePortGroup(String vendor, String hostName, PortGroupModel pgModel) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.updatePortGroup(hostName, pgModel);
    }

    public BocloudResult addHost(String vendor, SwitchModel model) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.addHost(model);
    }

    public BocloudResult listProfile(String vendor) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.listProfile();
    }

    public BocloudResult listByVdc(String vendor, String vdcName) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.listByVdc(vdcName);
    }

    public BocloudResult listMac(String vendor) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.listMac();
    }

    public BocloudResult addVlanGroup(String vendor, SangforVlanGroupModel model) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.addVlanGroup(model);
    }

    public BocloudResult editVlanGroup(String vendor, SangforVlanGroupModel model) {
        NetworkProvider provider = getProvider(vendor, NetworkProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.editVlanGroup(model);
    }
}
