package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.model.azure.network.NicIpConfigurationModel;
import com.bocloud.cmp.provider.network.NetworkInterfaceProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: NetworkInterfaceDriver
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/9/29 15:27
 */
@Service("networkInterfaceDriver")
public class NetworkInterfaceDriver extends GenericDriver<NetworkInterfaceProvider> {

    @Autowired
    private ProviderRepository repository;


    public BocloudResult remove(String vendor, String region, String netFaceId) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.remove(netFaceId);
    }

    public BocloudResult modify(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult vmChangeInterface(String vendor, String region, String netFaceId, String instanceId) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.vmChangeInterface(netFaceId, instanceId);
    }

    public BocloudResult changeSubnet(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.changeSubnet(model);
    }

    public BocloudResult create(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult attach(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.attach(model);
    }

    public BocloudResult detach(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detach(model);
    }

    public BocloudResult sgroups(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.sgroups(model);
    }

    public BocloudResult detail(String vendor, String region, String netFaceId) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detail(netFaceId);
    }

    public BocloudResult attachSecurityGroup(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.attachSecurityGroup(model);
    }

    public BocloudResult detachSecurityGroup(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detachSecurityGroup(model);
    }

    public BocloudResult getStatus(String vendor, String region, String nwiId, String status) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.getStatus(nwiId, status);
    }

    public BocloudResult assignPrivateIpAddresses(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.assignPrivateIpAddresses(model);
    }

    public BocloudResult disassignPrivateIpAddresses(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.disassignPrivateIpAddresses(model);
    }

    public BocloudResult disassignIpv6Addresses(String vendor, String region, String networkInterfaceUuid, List<String> ipv6Addresss) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.disassignIpv6Addresses(networkInterfaceUuid, ipv6Addresss);
    }

    public BocloudResult assignIpv6Addresses(String vendor, String region, String networkInterfaceUuid, List<String> ipv6Addresss,
                                             Integer ipv6AddressCount) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.assignIpv6Addresses(networkInterfaceUuid, ipv6Addresss, ipv6AddressCount);
    }

    public BocloudResult networkInterfaceAddNicIpConfiguration(String vendor, String region, NicIpConfigurationModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.networkInterfaceAddNicIpConfiguration(model);
    }

    public BocloudResult networkInterfaceModifyNicIpConfiguration(String vendor, String region, NicIpConfigurationModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.networkInterfaceModifyNicIpConfiguration(model);
    }

    public BocloudResult networkInterfaceRemoveNicIpConfiguration(String vendor, String region, NicIpConfigurationModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.networkInterfaceRemoveNicIpConfiguration(model);
    }

    public BocloudResult nicIpConfigurationAttachPublicIpAddress(String vendor, String region, NicIpConfigurationModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.nicIpConfigurationAttachPublicIpAddress(model);
    }

    public BocloudResult nicIpConfigurationdetachPublicIpAddress(String vendor, String region, NicIpConfigurationModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.nicIpConfigurationdetachPublicIpAddress(model);
    }

    public BocloudResult addDnsServer(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.addDnsServer(model);
    }

    public BocloudResult removeDnsServer(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.removeDnsServer(model);
    }

    public BocloudResult ipForwarding(String vendor, String region, NetworkInterfaceModel model) {
        NetworkInterfaceProvider provider = getProvider(vendor, NetworkInterfaceProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.ipForwarding(model);
    }


}
