package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.RouteTableModel;
import com.bocloud.cmp.provider.network.RouteTableProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * AWSRouteTable实现类
 *
 * @author
 */
@Service("routeTableDriver")
public class RouteTableDriver extends GenericDriver<RouteTableProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, String region) {
        RouteTableProvider provider = getProvider(vendor, RouteTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult create(RouteTableModel model, String vendor, String region) {
        RouteTableProvider provider = getProvider(vendor, RouteTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult delete(String routeTableId, String vendor, String region) {
        RouteTableProvider provider = getProvider(vendor, RouteTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.delete(routeTableId);
    }

    public BocloudResult update(RouteTableModel model, String vendor, String region) {
        RouteTableProvider provider = getProvider(vendor, RouteTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult getByVpc(String vpcUuid, String vendor, String region) {
        RouteTableProvider provider = getProvider(vendor, RouteTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.getByVpc(vpcUuid);
    }

    public BocloudResult unAssociate(String vendor, String region, String routeTableId, String vSwitchId) {
        RouteTableProvider provider = getProvider(vendor, RouteTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.unAssociate(routeTableId, vSwitchId);
    }

    public BocloudResult associate(String vendor, String region, String routeTableId, String vSwitchId) {
        RouteTableProvider provider = getProvider(vendor, RouteTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.associate(routeTableId, vSwitchId);
    }

    public BocloudResult associateSubnet(String vendor, String region, String routeTableId, List<String> subnetUuids) {
        RouteTableProvider provider = getProvider(vendor, RouteTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.associateSubnet(subnetUuids, routeTableId);
    }

    public BocloudResult unAssociateSubnet(String vendor, String region, String routeTableId, String subnetUuids) {
        RouteTableProvider provider = getProvider(vendor, RouteTableProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.unAssociateSubnet(subnetUuids, routeTableId);
    }

}
