package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.network.SecurityGroupProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("securityGroupDriver")
public class SecurityGroupDriver extends GenericDriver<SecurityGroupProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String region) {
		SecurityGroupProvider provider = getProvider(vendor, SecurityGroupProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, String region, SecurityGroupModel group) {
		SecurityGroupProvider provider = getProvider(vendor, SecurityGroupProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(group);
	}

	public BocloudResult create(String vendor, String username, String password, String projectId, String domainName,
			SecurityGroupModel group) {
		SecurityGroupProvider provider = getProvider(vendor, username, password, projectId, domainName,
				SecurityGroupProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(group);
	}

	public BocloudResult create(String vendor, String projectName, String region, SecurityGroupModel group) {
		SecurityGroupProvider provider = getProvider(vendor, projectName, SecurityGroupProvider.class, repository,
				region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(group);
	}

	public BocloudResult remove(String sgId, String vendor, String region) {
		SecurityGroupProvider provider = getProvider(vendor, SecurityGroupProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.remove(sgId);
	}

	public BocloudResult detail(String sgId, String vendor, String region) {
		SecurityGroupProvider provider = getProvider(vendor, SecurityGroupProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.detail(sgId);
	}

	public BocloudResult getProjectDefaultSecg(String vendor, String projectName) {
		SecurityGroupProvider provider = getProvider(vendor, projectName, SecurityGroupProvider.class, repository,
				null);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.getProjectDefaultSecg();
	}

	public BocloudResult modify(String vendor, String region, SecurityGroupModel model) {
		SecurityGroupProvider provider = getProvider(vendor, SecurityGroupProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.modify(model);
	}

	public BocloudResult getByVpc(String vpcId, String vendor, String region) {
		SecurityGroupProvider provider = getProvider(vendor, SecurityGroupProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.getByVpc(vpcId);
	}

	public BocloudResult remove(String vendor, SecurityGroupModel model) {
		SecurityGroupProvider provider = getProvider(vendor, SecurityGroupProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.remove(model);
	}
}
