package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.network.SecurityGroupRuleProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 安全组规则provider接口实现类
 *
 * @author weiwei
 */
@Service("securityGroupRuleDriver")
public class SecurityGroupRuleDriver extends GenericDriver<SecurityGroupRuleProvider> {
    @Autowired
    private ProviderRepository repository;

    public BocloudResult create(String vendor, String region, SecurityGroupRuleModel model) {
        SecurityGroupRuleProvider provider = getProvider(vendor, SecurityGroupRuleProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult create(String vendor, String projectName, String region, SecurityGroupRuleModel model) {
        SecurityGroupRuleProvider provider = getProvider(vendor, projectName, SecurityGroupRuleProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult remove(String ruleId, String vendor, String region) {
        SecurityGroupRuleProvider provider = getProvider(vendor, SecurityGroupRuleProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(ruleId);
    }

    public BocloudResult remove(String ruleId, String vendor, String projectName, String region) {
        SecurityGroupRuleProvider provider = getProvider(vendor, projectName, SecurityGroupRuleProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(ruleId);
    }

    public BocloudResult remove(SecurityGroupRuleModel ruleModel, String vendor, String region) {
        SecurityGroupRuleProvider provider = getProvider(vendor, SecurityGroupRuleProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(ruleModel);
    }

    public BocloudResult modify(String vendor, String region, SecurityGroupRuleModel model) {
        SecurityGroupRuleProvider provider = getProvider(vendor, SecurityGroupRuleProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult list(String vendor, String region, SecurityGroupRuleModel model) {
        SecurityGroupRuleProvider provider = getProvider(vendor, SecurityGroupRuleProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.list(model.getGroupUuid(),model.getVdcUri());
    }
}
