package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.network.SubnetProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("subnetDriver")
public class SubnetDriver extends GenericDriver<SubnetProvider> {
    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, String region) {
        SubnetProvider provider = getProvider(vendor, SubnetProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult create(String vendor, String region, SubnetModel subnet) {
        SubnetProvider provider = getProvider(vendor, SubnetProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(subnet);
    }

    public BocloudResult create(String vendor, SubnetModel subnet, String username, String password, String projectId,
                                String domainName) {
        SubnetProvider provider = getProvider(vendor, username, password, projectId, domainName, SubnetProvider.class,
                repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(subnet);
    }

    public BocloudResult create(String vendor, String projectName, String region, SubnetModel subnet) {
        SubnetProvider provider = getProvider(vendor, projectName, SubnetProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(subnet);
    }

    public BocloudResult modify(String vendor, String region, SubnetModel subnet) {
        SubnetProvider provider = getProvider(vendor, SubnetProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.modify(subnet);
    }

    public BocloudResult modify(String vendor, String region, SubnetModel subnet, String username, String password,
                                String projectId, String domainName) {
        SubnetProvider provider = getProvider(vendor, username, password, projectId, domainName, SubnetProvider.class,
                repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.modify(subnet);
    }

    public BocloudResult remove(String subnetId, String vendor, String region) {
        SubnetProvider provider = getProvider(vendor, SubnetProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(subnetId);
    }

    public BocloudResult remove(String subnetId, String vendor, String username, String password, String projectId,
                                String domainName) {
        SubnetProvider provider = getProvider(vendor, username, password, projectId, domainName, SubnetProvider.class,
                repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(subnetId);
    }

    public BocloudResult remove(SubnetModel subnet, String vendor, String region) {
        SubnetProvider provider = getProvider(vendor, SubnetProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(subnet);
    }

    public BocloudResult remove(SubnetModel subnet, String vendor, String username, String password, String projectId,
                                String domainName) {
        SubnetProvider provider = getProvider(vendor, username, password, projectId, domainName, SubnetProvider.class,
                repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(subnet);
    }

    public BocloudResult detail(String subnetId, String vendor, String region) {
        SubnetProvider provider = getProvider(vendor, SubnetProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.detail(subnetId);
    }

    public BocloudResult associate(String vendor, String region, SubnetModel subnetModel) {
        SubnetProvider provider = getProvider(vendor, SubnetProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.associate(subnetModel);
    }

    public BocloudResult release(String vendor, String region, SubnetModel subnetModel) {
        SubnetProvider provider = getProvider(vendor, SubnetProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.release(subnetModel);
    }

	public BocloudResult associateIpv6CidrBlock(String subnetId, String ipv6CidrBlock, String vendor, String region) {
		SubnetProvider provider = getProvider(vendor, SubnetProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.associateIpv6CidrBlock(subnetId, ipv6CidrBlock);
	}

	public BocloudResult disassociateCidrBlock(String associationId, String vendor, String region) {
		SubnetProvider provider = getProvider(vendor, SubnetProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.disassociateCidrBlock(associationId);
	}
}
