package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.LbTargetGroupModel;
import com.bocloud.cmp.provider.network.TargetGroupProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * AWS目标群组实现类
 *
 * @author
 */
@Service("targetGroupDriver")
public class TargetGroupDriver extends GenericDriver<TargetGroupProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String region) {
		TargetGroupProvider provider = getProvider(vendor, TargetGroupProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(LbTargetGroupModel model, String vendor, String region) {
		TargetGroupProvider provider = getProvider(vendor, TargetGroupProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String uuid, String vendor, String region) {
		TargetGroupProvider provider = getProvider(vendor, TargetGroupProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.delete(uuid);
	}

	public BocloudResult registerTargets(String uuid, Map<String, Integer> targetIdPorts, String vendor,
			String region) {
		TargetGroupProvider provider = getProvider(vendor, TargetGroupProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.registerTargets(uuid, targetIdPorts);
	}

	public BocloudResult deregisterTargets(String uuid, Map<String, Integer> targetIdPorts, String vendor,
			String region) {
		TargetGroupProvider provider = getProvider(vendor, TargetGroupProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "平台认证失败！");
		}
		return provider.deregisterTargets(uuid, targetIdPorts);
	}
}
