package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.VpnConnModel;
import com.bocloud.cmp.provider.network.VpnConnProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("vpnConnDriver")
public class VpnConnDriver extends GenericDriver<VpnConnProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, String region) {
        VpnConnProvider provider = getProvider(vendor, VpnConnProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult create(VpnConnModel model, String vendor, String projectName, String region) {
        VpnConnProvider provider = getProvider(vendor, projectName, VpnConnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult detail(String vpnConnUuid, String vendor, String region) {
        VpnConnProvider provider = getProvider(vendor, VpnConnProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.detail(vpnConnUuid);
    }

    public BocloudResult remove(String vpnConnUuid, String vendor, String region) {
        VpnConnProvider provider = getProvider(vendor, VpnConnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.delete(vpnConnUuid);
    }

    public BocloudResult modify(VpnConnModel model, String vendor, String region) {
        VpnConnProvider provider = getProvider(vendor, VpnConnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.modify(model);
    }

}
