package com.bocloud.cmp.driver.network;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.VpnModel;
import com.bocloud.cmp.model.VpnPolicyModel;
import com.bocloud.cmp.provider.network.VpnProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("vpnDriver")
public class VpnDriver extends GenericDriver<VpnProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, String region) {
        VpnProvider provider = getProvider(vendor, VpnProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult create(VpnModel model, String vendor, String projectName, String region) {
        VpnProvider provider = getProvider(vendor, projectName, VpnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult detail(String vpnUuid, String vendor, String region) {
        VpnProvider provider = getProvider(vendor, VpnProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.detail(vpnUuid);
    }

    public BocloudResult remove(String vpnUuid, String vendor, String region) {
        VpnProvider provider = getProvider(vendor, VpnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.delete(vpnUuid);
    }

    public BocloudResult modify(VpnModel model, String vendor, String region) {
        VpnProvider provider = getProvider(vendor, VpnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult createIkePolicy(VpnPolicyModel vpnPolicyModel, String vendor, String projectName, String region) {
        VpnProvider provider = getProvider(vendor, projectName, VpnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.createIkePolicy(vpnPolicyModel);
    }

    public BocloudResult modfiyIkePolicy(VpnPolicyModel vpnPolicyModel, String vendor, String region) {
        VpnProvider provider = getProvider(vendor, VpnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.modfiyIkePolicy(vpnPolicyModel);
    }

    public BocloudResult removeIkePolicy(String policyUuid, String vendor, String region) {
        VpnProvider provider = getProvider(vendor, VpnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.removeIkePolicy(policyUuid);
    }

    public BocloudResult createIPsecPolicy(VpnPolicyModel model, String vendor, String projectName, String region) {
        VpnProvider provider = getProvider(vendor, projectName, VpnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.createIPsecPolicy(model);
    }

    public BocloudResult modfiyIPsecPolicy(VpnPolicyModel vpnPolicyModel, String vendor, String region) {
        VpnProvider provider = getProvider(vendor, VpnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.modfiyIPsecPolicy(vpnPolicyModel);
    }

    public BocloudResult removeIPsecPolicy(String policyUuid, String vendor, String region) {
        VpnProvider provider = getProvider(vendor, VpnProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "云平台认证失败！");
        }
        return provider.removeIPsecPolicy(policyUuid);
    }

}
