package com.bocloud.cmp.driver.nsx;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.nsx.ComputeManagerProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.nsx.NsxComputeManagerModel;

/**
 * nsx计算管理器
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月24日
 *
 */
@Service("computeManagerDriver")
public class ComputeManagerDriver extends GenericDriver<ComputeManagerProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		ComputeManagerProvider provider = getProvider(vendor, ComputeManagerProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, NsxComputeManagerModel model) {
		ComputeManagerProvider provider = getProvider(vendor, ComputeManagerProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vendor, String managerId) {
		ComputeManagerProvider provider = getProvider(vendor, ComputeManagerProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(managerId);
	}

	public BocloudResult detail(String vendor, String managerId) {
		ComputeManagerProvider provider = getProvider(vendor, ComputeManagerProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(managerId);
	}

	public BocloudResult update(String vendor, NsxComputeManagerModel model) {
		ComputeManagerProvider provider = getProvider(vendor, ComputeManagerProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}
}
