package com.bocloud.cmp.driver.nsx;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.nsx.EdgeClusterProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.nsx.NsxEdgeClusterModel;

/**
 * nsx edge集群
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月24日
 *
 */
@Service("edgeClusterDriver")
public class EdgeClusterDriver extends GenericDriver<EdgeClusterProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		EdgeClusterProvider provider = getProvider(vendor, EdgeClusterProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, NsxEdgeClusterModel model) {
		EdgeClusterProvider provider = getProvider(vendor, EdgeClusterProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vendor, String switchId) {
		EdgeClusterProvider provider = getProvider(vendor, EdgeClusterProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(switchId);
	}

	public BocloudResult detail(String vendor, String clusterId) {
		EdgeClusterProvider provider = getProvider(vendor, EdgeClusterProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(clusterId);
	}

	public BocloudResult update(String vendor, NsxEdgeClusterModel model) {
		EdgeClusterProvider provider = getProvider(vendor, EdgeClusterProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}
}
