package com.bocloud.cmp.driver.nsx;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.nsx.FirewallSectionProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.nsx.NsxFirewallSectionModel;

/**
 * nsx防火墙区域
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月24日
 *
 */
@Service("firewallSectionDriver")
public class FirewallSectionDriver extends GenericDriver<FirewallSectionProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		FirewallSectionProvider provider = getProvider(vendor, FirewallSectionProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, NsxFirewallSectionModel model) {
		FirewallSectionProvider provider = getProvider(vendor, FirewallSectionProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vendor, String firewallId) {
		FirewallSectionProvider provider = getProvider(vendor, FirewallSectionProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(firewallId);
	}

	public BocloudResult detail(String vendor, String firewallId) {
		FirewallSectionProvider provider = getProvider(vendor, FirewallSectionProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(firewallId);
	}

	public BocloudResult update(String vendor, NsxFirewallSectionModel model) {
		FirewallSectionProvider provider = getProvider(vendor, FirewallSectionProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}

}
