package com.bocloud.cmp.driver.nsx;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.nsx.LbServiceProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.nsx.NsxLbServiceModel;

/**
 * nsx负载均衡器
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月24日
 *
 */
@Service("lbServiceDriver")
public class LbServiceDriver extends GenericDriver<LbServiceProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		LbServiceProvider provider = getProvider(vendor, LbServiceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, NsxLbServiceModel model) {
		LbServiceProvider provider = getProvider(vendor, LbServiceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vendor, String lbServiceId) {
		LbServiceProvider provider = getProvider(vendor, LbServiceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(lbServiceId);
	}

	public BocloudResult detail(String vendor, String lbServiceId) {
		LbServiceProvider provider = getProvider(vendor, LbServiceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(lbServiceId);
	}

	public BocloudResult update(String vendor, NsxLbServiceModel model) {
		LbServiceProvider provider = getProvider(vendor, LbServiceProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}
}
