package com.bocloud.cmp.driver.nsx;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.nsx.LogicalSwitchProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.nsx.NsxLogicalSwitchModel;

/**
 * nsx逻辑交换机
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月24日
 *
 */
@Service("logicalSwitchDriver")
public class LogicalSwitchDriver extends GenericDriver<LogicalSwitchProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		LogicalSwitchProvider provider = getProvider(vendor, LogicalSwitchProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, NsxLogicalSwitchModel model) {
		LogicalSwitchProvider provider = getProvider(vendor, LogicalSwitchProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vendor, String switchId) {
		LogicalSwitchProvider provider = getProvider(vendor, LogicalSwitchProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(switchId);
	}

	public BocloudResult detail(String vendor, String switchId) {
		LogicalSwitchProvider provider = getProvider(vendor, LogicalSwitchProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(switchId);
	}

	public BocloudResult update(String vendor, NsxLogicalSwitchModel model) {
		LogicalSwitchProvider provider = getProvider(vendor, LogicalSwitchProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}
}
