package com.bocloud.cmp.driver.nsx;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.nsx.NatProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.nsx.NsxNatRuleModel;

/**
 * nsx NAT
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月24日
 *
 */
@Service("natDriver")
public class NatDriver extends GenericDriver<NatProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String routerId) {
		NatProvider provider = getProvider(vendor, NatProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list(routerId);
	}

	public BocloudResult create(String vendor, NsxNatRuleModel model) {
		NatProvider provider = getProvider(vendor, NatProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vendor, String routerId, String natId) {
		NatProvider provider = getProvider(vendor, NatProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(routerId, natId);
	}

	public BocloudResult detail(String vendor, String routerId, String natId) {
		NatProvider provider = getProvider(vendor, NatProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(routerId, natId);
	}

	public BocloudResult update(String vendor, NsxNatRuleModel model) {
		NatProvider provider = getProvider(vendor, NatProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}
}
