package com.bocloud.cmp.driver.nsx;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.nsx.RouterPortProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.nsx.NsxRouterPortModel;

/**
 * nsx路由器端口
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月24日
 *
 */
@Service("routerPortDriver")
public class RouterPortDriver extends GenericDriver<RouterPortProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		RouterPortProvider provider = getProvider(vendor, RouterPortProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, NsxRouterPortModel model) {
		RouterPortProvider provider = getProvider(vendor, RouterPortProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vendor, String portId) {
		RouterPortProvider provider = getProvider(vendor, RouterPortProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(portId);
	}

	public BocloudResult detail(String vendor, String portId) {
		RouterPortProvider provider = getProvider(vendor, RouterPortProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(portId);
	}

	public BocloudResult update(String vendor, NsxRouterPortModel model) {
		RouterPortProvider provider = getProvider(vendor, RouterPortProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}
}
