package com.bocloud.cmp.driver.nsx;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.nsx.TransportZoneProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.nsx.NsxTransportZoneModel;

/**
 * nsx传输区域
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月24日
 *
 */
@Service("transportZoneDriver")
public class TransportZoneDriver extends GenericDriver<TransportZoneProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		TransportZoneProvider provider = getProvider(vendor, TransportZoneProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, NsxTransportZoneModel model) {
		TransportZoneProvider provider = getProvider(vendor, TransportZoneProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vendor, String zoneId) {
		TransportZoneProvider provider = getProvider(vendor, TransportZoneProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(zoneId);
	}

	public BocloudResult detail(String vendor, String zoneId) {
		TransportZoneProvider provider = getProvider(vendor, TransportZoneProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(zoneId);
	}

	public BocloudResult update(String vendor, NsxTransportZoneModel model) {
		TransportZoneProvider provider = getProvider(vendor, TransportZoneProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}
}
