package com.bocloud.cmp.driver.nsx;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.nsx.VirtualServerProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.nsx.NsxVirtualServerModel;

/**
 * nsx虚拟服务器
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月24日
 *
 */
@Service("virtualServerDriver")
public class VirtualServerDriver extends GenericDriver<VirtualServerProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor) {
		VirtualServerProvider provider = getProvider(vendor, VirtualServerProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, NsxVirtualServerModel model) {
		VirtualServerProvider provider = getProvider(vendor, VirtualServerProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vendor, String lbServiceId) {
		VirtualServerProvider provider = getProvider(vendor, VirtualServerProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(lbServiceId);
	}

	public BocloudResult detail(String vendor, String lbServiceId) {
		VirtualServerProvider provider = getProvider(vendor, VirtualServerProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(lbServiceId);
	}

	public BocloudResult update(String vendor, NsxVirtualServerModel model) {
		VirtualServerProvider provider = getProvider(vendor, VirtualServerProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}
}
