package com.bocloud.cmp.driver.sdn;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.cisco.EpgModel;
import com.bocloud.cmp.provider.network.EpgProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.agile.sfc.AgileEpgModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("epgDriver")
public class EpgDriver extends GenericDriver<EpgProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult create(String vendor, EpgModel model) {
        EpgProvider provider = getProvider(vendor, EpgProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult remove(String vendor, String dn) {
        EpgProvider provider = getProvider(vendor, EpgProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.remove(dn);
    }

    public BocloudResult<AgileEpgModel> create(String vendor, String region, AgileEpgModel model) {
        EpgProvider provider = getProvider(vendor, EpgProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult<AgileEpgModel> modify(String vendor, String region, AgileEpgModel model) {
        EpgProvider provider = getProvider(vendor, EpgProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult<AgileEpgModel> detail(String vendor, String region, String uuid) {
        EpgProvider provider = getProvider(vendor, EpgProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.detail(uuid);
    }


}
