package com.bocloud.cmp.driver.sdn;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.cisco.TenantModel;
import com.bocloud.cmp.provider.network.TenantProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.agile.AgileTenantModel;
import com.bocloud.nms.model.agile.AgileTenantStatisticsModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service("tenantDriver")
public class TenantDriver extends GenericDriver<TenantProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult create(String vendor, TenantModel model) {
        TenantProvider provider = getProvider(vendor, TenantProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult<?> remove(String vendor, String dn) {
        TenantProvider provider = getProvider(vendor, TenantProvider.class, repository);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.remove(dn);
    }

    public BocloudResult<AgileTenantModel> create(String vendor, String region, AgileTenantModel model) {
        TenantProvider provider = getProvider(vendor, TenantProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult<AgileTenantModel> modify(String vendor, String region, AgileTenantModel model) {
        TenantProvider provider = getProvider(vendor, TenantProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult<AgileTenantModel> detail(String vendor, String region, String uuid) {
        TenantProvider provider = getProvider(vendor, TenantProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.detail(uuid);
    }

    public BocloudResult<?> remove(String vendor, String region, String uuid) {
        TenantProvider provider = getProvider(vendor, TenantProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.remove(uuid);
    }

    public BocloudResult<AgileTenantStatisticsModel> statistics(String vendor, String region, String uuid) {
        TenantProvider provider = getProvider(vendor, TenantProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.statistics(uuid);
    }

    public BocloudResult<List<String>> queryRouterStatus(String vendor, String region, String uuid, String queryType) {
        TenantProvider provider = getProvider(vendor, TenantProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.queryRouterStatus(uuid, queryType);
    }

    public BocloudResult<List<String>> querySwitchStatus(String vendor, String region, String uuid, String queryType) {
        TenantProvider provider = getProvider(vendor, TenantProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return new BocloudResult<>(false, "平台认证失败！");
        }
        return provider.querySwitchStatus(uuid, queryType);
    }


}
