package com.bocloud.cmp.driver.secuirty;

import com.bocloud.cmp.model.sangfor.SangforFirewallRuleModel;
import com.bocloud.cmp.params.Sangfor.SangforFireWallParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.FirewallModel;
import com.bocloud.cmp.model.FirewallPolicyModel;
import com.bocloud.cmp.model.FirewallRuleModel;
import com.bocloud.cmp.provider.security.FirewallProvider;
import com.bocloud.common.model.BocloudResult;

@Service("firewallDriver")
public class FirewallDriver extends GenericDriver<FirewallProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String region) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, String region, FirewallModel firewall) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(firewall);
	}

	public BocloudResult createV68(String vendor, String region, SangforFireWallParams firewall) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createV68(firewall);
	}

	public BocloudResult updateV68(String vendor, String region, SangforFireWallParams firewall) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.updateV68(firewall);
	}

	public BocloudResult create(String vendor, String username, String password, String projectId, String domainName,
			FirewallModel firewall) {
		FirewallProvider provider = getProvider(vendor, username, password, projectId, domainName,
				FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(firewall);
	}

	public BocloudResult create(String vendor, String projectName, String region, FirewallModel firewall) {
		FirewallProvider provider = getProvider(vendor, projectName, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(firewall);
	}

	public BocloudResult remove(String firewallId, String vendor, String region) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(firewallId);
	}

	public BocloudResult update(String vendor, String region, FirewallModel firewall) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(firewall);
	}

	public BocloudResult relevanceRouter(String vendor, String region, FirewallModel firewall) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.relevanceRouter(firewall);
	}

	public BocloudResult detail(String firewallId, String vendor, String region) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(firewallId);
	}

	public BocloudResult listPolicy(String vendor) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.listPolicy();
	}

	public BocloudResult createPolicy(String vendor, FirewallPolicyModel policyModel) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createPolicy(policyModel);
	}

	public BocloudResult createPolicy(String vendor, String projectName, FirewallPolicyModel policyModel) {
		FirewallProvider provider = getProvider(vendor, projectName, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createPolicy(policyModel);
	}

	public BocloudResult insertRule(String vendor, String policyId, String ruleId, String type, String referRuleId) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.insertRule(policyId, ruleId, type, referRuleId);
	}

	public BocloudResult removePolicy(String vendor, String policyId) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removePolicy(policyId);
	}

	public BocloudResult removeRule(String vendor, String policyId, String ruleId) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeRule(policyId, ruleId);
	}

	public BocloudResult updatePolicy(FirewallPolicyModel policyModel, String vendor) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.updatePolicy(policyModel);
	}

	public BocloudResult detailPolicy(String policyId, String vendor) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detailPolicy(policyId);
	}

	public BocloudResult listRule(String vendor) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.listRule();
	}

	public BocloudResult createRule(FirewallRuleModel ruleModel, String vendor) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createRule(ruleModel);
	}

	public BocloudResult createRule(FirewallRuleModel ruleModel, String vendor, String projectName) {
		FirewallProvider provider = getProvider(vendor, projectName, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createRule(ruleModel);
	}

	public BocloudResult removeRule(String ruleId, String vendor) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeRule(ruleId);
	}

	public BocloudResult updateRule(FirewallRuleModel ruleModel, String vendor) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.updateRule(ruleModel);
	}

	public BocloudResult detailRule(String ruleId, String vendor) {
		FirewallProvider provider = getProvider(vendor, FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detailRule(ruleId);
	}

	public BocloudResult createRule(String vendor, String username, String password, String projectId,
			String domainName, FirewallRuleModel model) {
		FirewallProvider provider = getProvider(vendor, username, password, projectId, domainName,
				FirewallProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.createRule(model);
	}

}
