package com.bocloud.cmp.driver.secuirty;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.security.KeypairProvider;
import com.bocloud.common.model.BocloudResult;

@Service("keypairDriver")
public class KeypairDriver extends GenericDriver<KeypairProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String region) {
		KeypairProvider provider = getProvider(vendor, KeypairProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String keypairName, String publicKey, String vendor, String region) {
		KeypairProvider provider = getProvider(vendor, KeypairProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(keypairName, publicKey);
	}

	public BocloudResult create(String keypairName, String publicKey, String vendor, String username, String password,
			String projectId, String domainName) {
		KeypairProvider provider = getProvider(vendor, username, password, projectId, domainName, KeypairProvider.class,
				repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(keypairName, publicKey);
	}

	public BocloudResult createByProject(String keypairName, String publicKey, String vendor, String projectName,
			String region) {
		KeypairProvider provider = getProvider(vendor, projectName, KeypairProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(keypairName, publicKey);
	}

	public BocloudResult create(String keypairName, String publicKey, String encryptMethod, String vendor,
			String region) {
		KeypairProvider provider = getProvider(vendor, KeypairProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(keypairName, publicKey, encryptMethod);
	}

	public BocloudResult detail(String keypairName, String vendor, String region) {
		KeypairProvider provider = getProvider(vendor, KeypairProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(keypairName);
	}

	public BocloudResult remove(String keypairName, String vendor, String region) {
		KeypairProvider provider = getProvider(vendor, KeypairProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.delete(keypairName);
	}

	public BocloudResult remove(String keypairName, String vendor, String username, String password, String projectId,
			String domainName) {
		KeypairProvider provider = getProvider(vendor, username, password, projectId, domainName, KeypairProvider.class,
				repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.delete(keypairName);
	}

	public BocloudResult modify(KeypairModel keypairModel, String vendor, String region) {
		KeypairProvider provider = getProvider(vendor, KeypairProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.modify(keypairModel);
	}

	public BocloudResult create(KeypairModel model, String vendor, String region) {
		KeypairProvider provider = getProvider(vendor, KeypairProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult delete(KeypairModel model, String vendor,
								String region) {
		KeypairProvider provider = getProvider(vendor, KeypairProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.delete(model);
	}

}
