package com.bocloud.cmp.driver.secuirty;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.network.VpcProvider;
import com.bocloud.common.model.BocloudResult;

@Service("vpcDriver")
public class VpcDriver extends GenericDriver<VpcProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult create(VpcModel model, String vendor) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult create(VpcModel model, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult remove(String vpcId, String vendor) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(vpcId);
	}

	public BocloudResult remove(String vpcId, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.remove(vpcId);
	}

	public BocloudResult list(String vendor) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult detail(String vpcId, String vendor, String regionId) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.detail(vpcId, regionId);
	}

	public BocloudResult update(VpcModel model, String vendor) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}

	public BocloudResult update(VpcModel model, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.update(model);
	}

	public BocloudResult associateCidrBlock(String vpcId, String cidrBlock, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.associateCidrBlock(vpcId, cidrBlock);
	}

	public BocloudResult associateIpv6CidrBlock(String vpcId, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.associateIpv6CidrBlock(vpcId);
	}

	public BocloudResult disassociateCidrBlock(String associationId, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.disassociateCidrBlock(associationId);
	}

	public BocloudResult disassociateVpcCidrBlock(String vpcId, String cidrBlock, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.disassociateVpcCidrBlock(vpcId, cidrBlock);
	}

	public BocloudResult withDdos(String vpcId, String planId, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.withDdos(vpcId, planId);
	}

	public BocloudResult withoutDdos(String vpcId, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.withoutDdos(vpcId);
	}

	public BocloudResult addAddressSpace(VpcModel model, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.addAddressSpace(model);
	}

	public BocloudResult removeAddressSpace(VpcModel model, String vendor, String region) {
		VpcProvider provider = getProvider(vendor, VpcProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return new BocloudResult(false, "云平台认证失败！");
		}
		return provider.removeAddressSpace(model);
	}
}
