package com.bocloud.cmp.driver.storage;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.BucketModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.storage.BucketProvider;
import com.bocloud.common.model.BocloudResult;

@Service("bucketDriver")
public class BucketDriver extends GenericDriver<BucketProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult create(String name, boolean share, String vendor, String region) {
		BucketProvider provider = getProvider(vendor, BucketProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.create(name, share);
	}

	public BocloudResult create(BucketModel model, String vendor, String region) {
		BucketProvider provider = getProvider(vendor, BucketProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.create(model);
	}

	public BocloudResult exists(String name, String vendor, String region) {
		BucketProvider provider = getProvider(vendor, BucketProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.exists(name);
	}

	public BocloudResult remove(BucketModel model, String vendor, String region) {
		BucketProvider provider = getProvider(vendor, BucketProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.remove(model);
	}

	public BocloudResult remove(String name, String vendor, String region) {
		BucketProvider provider = getProvider(vendor, BucketProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.remove(name);
	}

	public BocloudResult detail(String name, String vendor, String region) {
		BucketProvider provider = getProvider(vendor, BucketProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.detail(name);
	}

	public BocloudResult detail(Long bucketId, String vendor) {
		BucketProvider provider = getProvider(vendor, BucketProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.detail(bucketId);
	}

	public BocloudResult list(String vendor, String region) {
		BucketProvider provider = getProvider(vendor, BucketProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult list(Butler butler) {
		BucketProvider provider = getProvider(butler, BucketProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult acl(BucketModel model, String vendor) {
		BucketProvider provider = getProvider(vendor, BucketProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.acl(model);
	}

	public BocloudResult acl(String bucket, boolean acl, String vendor, String region) {
		BucketProvider provider = getProvider(vendor, BucketProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.acl(bucket, acl);
	}

	public BocloudResult remove(String name, Butler butler) {
		BucketProvider provider = getProvider(butler, BucketProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.remove(name);
	}

}
