package com.bocloud.cmp.driver.storage;

import java.io.InputStream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.BucketObjectModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.storage.BucketObjectProvider;
import com.bocloud.common.model.BocloudResult;

@Service("bucketObjectDriver")
public class BucketObjectDriver extends GenericDriver<BucketObjectProvider> {
	@Autowired
	private ProviderRepository repository;

	public BocloudResult listItem(String bucketName, String vendor) {
		BucketObjectProvider provider = getProvider(vendor,
				BucketObjectProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.list(bucketName);
	}

	public BocloudResult remove(BucketObjectModel model, String vendor, String regionId) {
		BucketObjectProvider provider = getProvider(vendor, BucketObjectProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.remove(model);
	}
	
	public BocloudResult remove(BucketObjectModel model, String vendor) {
		BucketObjectProvider provider = getProvider(vendor, BucketObjectProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.remove(model);
	}

	public BocloudResult detail(BucketObjectModel model, String vendor) {
		BucketObjectProvider provider = getProvider(vendor,
				BucketObjectProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.detail(model);
	}

	public InputStream download(BucketObjectModel model, String vendor) {
		BucketObjectProvider provider = getProvider(vendor,
				BucketObjectProvider.class, repository);
		if (before(provider).isFailed()) {
			return null;
		}
		return provider.download(model);
	}
	
	public InputStream download(BucketObjectModel model, String vendor, String regionId) {
		BucketObjectProvider provider = getProvider(vendor, BucketObjectProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return null;
		}
		return provider.download(model);
	}

	public InputStream download(BucketObjectModel model, Butler butler) {
		BucketObjectProvider provider = getProvider(butler,
				BucketObjectProvider.class, repository);
		if (before(provider).isFailed()) {
			return null;
		}
		return provider.download(model);
	}

	public BocloudResult upload(BucketObjectModel model, String vendor, String regionId) {
		BucketObjectProvider provider = getProvider(vendor, BucketObjectProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.upload(model);
	}

	public BocloudResult acl(BucketObjectModel model, String vendor) {
		BucketObjectProvider provider = getProvider(vendor,
				BucketObjectProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.acl(model);
	}
	
	public BocloudResult acl(BucketObjectModel model, String vendor, String regionId) {
		BucketObjectProvider provider = getProvider(vendor, BucketObjectProvider.class, repository, regionId);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.acl(model);
	}

	public BocloudResult acl(BucketObjectModel model, Butler butler) {
		BucketObjectProvider provider = getProvider(butler,
				BucketObjectProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.acl(model);
	}

	public BocloudResult upload(BucketObjectModel model, Butler butler) {
		BucketObjectProvider provider = getProvider(butler,
				BucketObjectProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.upload(model);
	}

	public BocloudResult remove(BucketObjectModel model, Butler butler) {
		BucketObjectProvider provider = getProvider(butler,
				BucketObjectProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.remove(model);
	}
}
