package com.bocloud.cmp.driver.storage;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.SanFabricZoneModel;
import com.bocloud.cmp.provider.storage.FabricZoneProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 光纤交换机zone操作类
 *
 * @author lkx007
 */
@Service("fabricZoneDriver")
public class FabricZoneDriver extends GenericDriver<FabricZoneProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, String fabricWwn) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.list(fabricWwn);
    }

    public BocloudResult create(String vendor, String fabricWwn, List<SanFabricZoneModel> zones) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.create(fabricWwn, zones);
    }

    public BocloudResult remove(String vendor, String fabricWwn, List<String> zones) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.remove(fabricWwn, zones);
    }

    public BocloudResult detail(String vendor, String fabricWwn, String zoneName) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detail(fabricWwn, zoneName);
    }

    public BocloudResult listAlias(String vendor, String fabricWwn) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.listAlias(fabricWwn);
    }

    public BocloudResult createAlias(String vendor, String fabricWwn, List<SanFabricZoneModel> aliases) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.createAlias(fabricWwn, aliases);
    }

    public BocloudResult removeAlias(String vendor, String fabricWwn, List<String> aliases) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.removeAlias(fabricWwn, aliases);
    }

    public BocloudResult detailAlias(String vendor, String fabricWwn, String alias) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detailAlias(fabricWwn, alias);
    }

    public BocloudResult create(String vendor, SanFabricZoneModel zone) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.create(zone);
    }

    public BocloudResult modify(String vendor, SanFabricZoneModel zone) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.modify(zone);
    }


    public BocloudResult createAlias(String vendor, SanFabricZoneModel zone) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.createAlias(zone);
    }

    public BocloudResult modifyAlias(String vendor, SanFabricZoneModel zone) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.modifyAlias(zone);
    }

    public BocloudResult remove(String vendor, SanFabricZoneModel zone) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.remove(zone);
    }

    public BocloudResult removeAlias(String vendor, SanFabricZoneModel zone) {
        FabricZoneProvider provider = getProvider(vendor, FabricZoneProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.removeAlias(zone);
    }
}
