package com.bocloud.cmp.driver.storage;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.xsky.FileUserGroupModel;
import com.bocloud.cmp.model.xsky.FileUserModel;
import com.bocloud.cmp.provider.storage.FileUserProvider;
import com.bocloud.common.model.BocloudResult;

/**
 * xsky 文件用户
 * 
 * @author admin
 *
 */
@Service("fileUserDriver")
public class FileUserDriver extends GenericDriver<FileUserProvider> {

	@Autowired
	private ProviderRepository repository;

	public BocloudResult create(FileUserModel fileUserModel, String vendor) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.create(fileUserModel);
	}

	public BocloudResult remove(String uuid, String vendor) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.remove(uuid);
	}

	public BocloudResult modify(FileUserModel model, String vendor) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.modify(model);
	}

	public BocloudResult list(String vendor) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult listUserGroups(String vendor) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.listUserGroups();
	}

	public BocloudResult createUserGroup(String vendor, FileUserGroupModel model) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.createUserGroup(model);
	}
	
	public BocloudResult modifyUserGroup(String vendor, FileUserGroupModel model) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.modifyUserGroup(model);
	}

	public BocloudResult removeUserGroup(String vendor, String uuid) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.removeUserGroup(uuid);
	}

	public BocloudResult removeGroupMember(String vendor, FileUserGroupModel model) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.removeGroupMember(model);
	}

	public BocloudResult addGroupMember(String vendor, FileUserGroupModel model) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.addGroupMember(model);
	}

	public BocloudResult updateUserGroup(String vendor, FileUserGroupModel model) {
		FileUserProvider provider = getProvider(vendor, FileUserProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.updateUserGroup(model);
	}

}
