package com.bocloud.cmp.driver.storage;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.ObjectStorageContainerModel;
import com.bocloud.cmp.model.ObjectStorageItemModel;
import com.bocloud.cmp.provider.storage.ObjectStorageProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;

@Service("objectStorageDriver")
public class ObjectStorageDriver extends GenericDriver<ObjectStorageProvider> {
    @Autowired
    private ProviderRepository repository;

    public BocloudResult listContainer(String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.listContainer();
    }


    public BocloudResult create(ObjectStorageContainerModel containerModel, String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.createContainer(containerModel);
    }


    public BocloudResult exists(ObjectStorageContainerModel container, String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.existsContainer(container);
    }


    public BocloudResult remove(ObjectStorageContainerModel container, String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.removeContainer(container);
    }


    public BocloudResult detail(ObjectStorageContainerModel container, String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detailContainer(container);
    }


    public BocloudResult listItem(String containerName, String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.listItem(containerName);
    }


    public BocloudResult create(ObjectStorageItemModel itemModel, String vendor, String region, String invokerPath) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.createItem(itemModel, invokerPath);
    }


    public BocloudResult exists(ObjectStorageItemModel itemModel, String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.existsItem(itemModel);
    }


    public BocloudResult remove(ObjectStorageItemModel itemModel, String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.removeItem(itemModel);
    }


    public BocloudResult detail(ObjectStorageItemModel itemModel, String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detailItem(itemModel);
    }


    public InputStream download(ObjectStorageItemModel itemModel, String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return null;
        }
        return provider.download(itemModel);
    }


    public BocloudResult upload(ObjectStorageItemModel itemModel, InputStream inputStream, String vendor, String region) {
        ObjectStorageProvider provider = getProvider(vendor, ObjectStorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.uploadItem(itemModel, inputStream);
    }
}
