package com.bocloud.cmp.driver.storage;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.OsdModel;
import com.bocloud.cmp.provider.storage.OsdProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * osd服务接口实现类
 *
 * @author weiwei
 */
@Service("osdDriver")
public class OsdDriver extends GenericDriver<OsdProvider> {
    @Autowired
    private ProviderRepository repository;

    public BocloudResult create(String vendor, OsdModel osdModel, String target) {
        OsdProvider provider = getProvider(vendor, OsdProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.create(osdModel);
    }

    public BocloudResult remove(String vendor, OsdModel osdModel, String target) {
        OsdProvider provider = getProvider(vendor, OsdProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.remove(osdModel);
    }


    public BocloudResult start(String vendor, String osdId, String target) {
        OsdProvider provider = getProvider(vendor, OsdProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.start(osdId, target);
    }


    public BocloudResult stop(String vendor, String osdId, String target) {
        OsdProvider provider = getProvider(vendor, OsdProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.stop(osdId, target);
    }


    public BocloudResult restart(String vendor, String osdId, String target) {
        OsdProvider provider = getProvider(vendor, OsdProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.restart(osdId, target);
    }
}
