package com.bocloud.cmp.driver.storage;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.StorageModel;
import com.bocloud.cmp.provider.storage.StorageProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 存储对象接口实现类
 *
 * @author weiwei
 */
@Service("storageDriver")
public class StorageDriver extends GenericDriver<StorageProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor) {
        StorageProvider provider = getProvider(vendor, StorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult get(String vendor, String storageUuid) {
        StorageProvider provider = getProvider(vendor, StorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.get(storageUuid);
    }

    public BocloudResult connect(String vendor, StorageModel model) {
        StorageProvider provider = getProvider(vendor, StorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.connect(model);
    }

    public BocloudResult create(String vendor, StorageModel model) {
        StorageProvider provider = getProvider(vendor, StorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult modify(String vendor, StorageModel model) {
        StorageProvider provider = getProvider(vendor, StorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult remove(String vendor, String host) {
        StorageProvider provider = getProvider(vendor, StorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.remove(host);
    }

    public BocloudResult detail(String vendor, String host) {
        StorageProvider provider = getProvider(vendor, StorageProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detail(host);
    }

}
