package com.bocloud.cmp.driver.storage;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.storage.StorageGroupModel;
import com.bocloud.cmp.provider.storage.StorageGroupProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service("storageGroupDriver")
public class StorageGroupDriver extends GenericDriver<StorageGroupProvider> {
    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, String region) {
        StorageGroupProvider provider = getProvider(vendor, StorageGroupProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.list();
    }


    public BocloudResult create(String vendor, String region, StorageGroupModel storageGroup) {
        StorageGroupProvider provider = getProvider(vendor, StorageGroupProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.create(storageGroup);
    }


    public BocloudResult remove(String vendor, String region, StorageGroupModel storageGroup) {
        StorageGroupProvider provider = getProvider(vendor, StorageGroupProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.remove(storageGroup);
    }

    public BocloudResult detail(String groupId, String vendor, String region) {
        StorageGroupProvider provider = getProvider(vendor, StorageGroupProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detail(groupId);
    }


    public BocloudResult modify(String vendor, String region, StorageGroupModel storageGroup) {
        StorageGroupProvider provider = getProvider(vendor, StorageGroupProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.modify(storageGroup);
    }

    public BocloudResult addMembers(String vendor, String region, StorageGroupModel storageGroup) {
        StorageGroupProvider provider = getProvider(vendor, StorageGroupProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.addMembers(storageGroup);
    }

    public BocloudResult removeMembers(String vendor, String region, StorageGroupModel storageGroup) {
        StorageGroupProvider provider = getProvider(vendor, StorageGroupProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.removeMembers(storageGroup);
    }


}
