package com.bocloud.cmp.driver.storage;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.DataStoreModel;
import com.bocloud.cmp.model.h3c.H3cStoragePoolModel;
import com.bocloud.cmp.provider.storage.StoragePoolProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * Scp查询交换机接口实现类
 *
 * @author weiwei
 */
@Service("storagePoolDriver")
public class StoragePoolDriver extends GenericDriver<StoragePoolProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult create(String vendor, DataStoreModel dataStoreModel) {
        StoragePoolProvider storagePoolProvider = getProvider(vendor, StoragePoolProvider.class, repository);
        if (before(storagePoolProvider).isFailed()) {
            return new BocloudResult(false, "用户鉴权失败！");
        } else {
            H3cStoragePoolModel storagePoolModel = new H3cStoragePoolModel();
            storagePoolModel.setName(dataStoreModel.getName());
            storagePoolModel.setHostId(dataStoreModel.getHostId());
            storagePoolModel.setHostName(dataStoreModel.getHostName());
            storagePoolModel.setType("dir");
            storagePoolModel.setSrcPath("/vms/" + dataStoreModel.getName());
            storagePoolModel.setAutoStart(true);
            return storagePoolProvider.create(storagePoolModel);
        }
    }

    public BocloudResult start(String vendor, DataStoreModel dataStoreModel) {
        StoragePoolProvider storagePoolProvider = getProvider(vendor, StoragePoolProvider.class, repository);
        if (before(storagePoolProvider).isFailed()) {
            return new BocloudResult(false, "用户鉴权失败！");
        } else {
            H3cStoragePoolModel storagePoolModel = new H3cStoragePoolModel();
            storagePoolModel.setName(dataStoreModel.getName());
            storagePoolModel.setHostId(dataStoreModel.getHostId());
            storagePoolModel.setHostName(dataStoreModel.getHostName());
            return storagePoolProvider.start(storagePoolModel);
        }
    }

    public BocloudResult detail(String vendor, DataStoreModel dataStoreModel) {
        StoragePoolProvider storagePoolProvider = getProvider(vendor, StoragePoolProvider.class, repository);
        if (before(storagePoolProvider).isFailed()) {
            return new BocloudResult(false, "用户鉴权失败！");
        } else {
            return storagePoolProvider.detail(dataStoreModel.getHostId().toString(), dataStoreModel.getName());
        }
    }

}
