package com.bocloud.cmp.driver.storage;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.SwitcherModel;
import com.bocloud.cmp.provider.storage.SwitcherProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 已有SwitcherDriver，但是使用的泛型是SwithProvider
 * 所以建立SwitchDriver，使用泛型SwitcherProvider
 *
 * 另外在compute包里面也有个SwitchDriver，使用泛型compute包的SwitchProvider
 */
@Service("storageSwitcherDriver")
public class SwitchDriver extends GenericDriver<SwitcherProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor) {
        SwitcherProvider provider = getProvider(vendor, SwitcherProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult detail(String vendor, String uuid) {
        SwitcherProvider provider = getProvider(vendor, SwitcherProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.detail(uuid);
    }

    public BocloudResult create(String vendor, SwitcherModel model) {
        SwitcherProvider provider = getProvider(vendor, SwitcherProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult modify(String vendor, SwitcherModel model) {
        SwitcherProvider provider = getProvider(vendor, SwitcherProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult remove(String vendor, SwitcherModel model) {
        SwitcherProvider provider = getProvider(vendor, SwitcherProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.remove(model);
    }
}
