package com.bocloud.cmp.driver.storage;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeMetadataModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.VolumeTypeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.model.storage.StorageGroupModel;
import com.bocloud.cmp.provider.storage.VolumeProvider;
import com.bocloud.common.model.BocloudResult;

@Service("volumeDriver")
public class VolumeDriver extends GenericDriver<VolumeProvider> {
	@Autowired
	private ProviderRepository repository;

	public BocloudResult list(String vendor, String region) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.list();
	}

	public BocloudResult create(String vendor, String region, VolumeModel volume) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.create(volume);
	}
	
	public BocloudResult create(String vendor, String username, String password, String projectId, String domainName,
			VolumeModel volume) {
		VolumeProvider provider = getProvider(vendor, username, password, projectId, domainName, VolumeProvider.class,
				repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.create(volume);
	}

	public BocloudResult create(String vendor, String projectName, String region, VolumeModel volume) {
		VolumeProvider provider = getProvider(vendor, projectName, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.create(volume);
	}

	public BocloudResult remove(String volumeId, String vendor, String region) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.remove(volumeId);
	}
	
	public BocloudResult remove(H3cVolumeModel volumeModel, String vendor) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, null);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.remove(volumeModel);
	}

	public BocloudResult remove(VolumeModel volume, String vendor, String region) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.remove(volume);
	}

	public BocloudResult detail(String volumeId, String vendor, String region) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.detail(volumeId);
	}

	public BocloudResult detail(String volumeId, String vendor, String username, String password, String projectId,
			String domainName) {
		VolumeProvider provider = getProvider(vendor, username, password, projectId, domainName, VolumeProvider.class,
				repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.detail(volumeId);
	}

	public BocloudResult modify(String vendor, String region, VolumeModel volume) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.modify(volume);
	}

	public BocloudResult migrateVolume(String vendor, String region, VolumeModel volume) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.migrateVolume(volume);
	}

	public BocloudResult copyVolume(String vendor, String region, VolumeModel volume) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.copyVolume(volume);
	}

	public BocloudResult snapshot(SnapshotModel model, String vendor, String region) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.snapshot(model);
	}

	public BocloudResult snapshot(SnapshotModel model, String vendor, String projectName, String region) {
		VolumeProvider provider = getProvider(vendor, projectName, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.snapshot(model);
	}

	public BocloudResult attach(String vendor, String region, String serverId, String volumeId) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.attach(serverId, volumeId);
	}

	public BocloudResult detach(String vendor, String region, String serverId, String volumeId) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.detach(serverId, volumeId);
	}

	public BocloudResult resetStatus(String volumeId, String vendor, String region, String status) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.resetStatus(volumeId, status);
	}

	public BocloudResult uploadToImage(String volumeId, ImageModel imageModel, String vendor, String region) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.uploadToImage(volumeId, imageModel);
	}

	public BocloudResult extend(String volumeId, Integer newSize, String vendor, String region) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.extend(volumeId, newSize);
	}
	
	public BocloudResult extend(String volumeId, Integer newSize, String type, String vendor, String region) {
        VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.extend(volumeId, newSize, type);
    }

	public BocloudResult extend(String volumeId, Integer newSize, String vendor, String username, String password,
			String projectId, String domainName) {
		VolumeProvider provider = getProvider(vendor, username, password, projectId, domainName, VolumeProvider.class,
				repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.extend(volumeId, newSize);
	}

	public BocloudResult attachByPoint(String vendor, String region, String serverId, String volumeId,
			String mountpoint) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.attach(serverId, mountpoint, volumeId);
	}

	public BocloudResult onboard(String vendor, String storageName, List<String> volumeIds) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.onboard(storageName, volumeIds);
	}

	public BocloudResult unmanage(String vendor, String storageName, List<String> volumeIds) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.unmanage(storageName, volumeIds);
	}

	public BocloudResult allVolumes(String vendor, String storageName) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.allVolumes(storageName);
	}

	public BocloudResult getMetadata(String vendor, String volumeUuid) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.getMetadata(volumeUuid);
	}

	public BocloudResult setMetadata(String vendor, String volumeUuid, VolumeMetadataModel metadataModel) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.setMetadata(volumeUuid, metadataModel);
	}

	public BocloudResult listMirrors(String vendor, String volumeUuid) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.listMirrors(volumeUuid);
	}

	public BocloudResult addGroups(String volumeKeyPath, List<StorageGroupModel> model, String vendor) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.addGroups(volumeKeyPath, model);
	}

	public BocloudResult removeGroups(String volumeKeyPath, List<StorageGroupModel> model, String vendor) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.removeGroups(volumeKeyPath, model);
	}

	public BocloudResult createVolumeType(VolumeTypeModel model, String vendor) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.createVolumeType(model);
	}
	
	public BocloudResult attach(String vendor, String region, VolumeModel model) {
        VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.attach(model);
    }

	public BocloudResult createVolumeType(VolumeTypeModel model, String vendor, String projectName) {
		VolumeProvider provider = getProvider(vendor, projectName, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.createVolumeType(model);
	}

	public BocloudResult deleteVolumeType(String volumeTypeUuid, String vendor) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.deleteVolumeType(volumeTypeUuid);
	}

	 public BocloudResult createTag(String vendor, String region, VolumeModel model) {
        VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.createTag(model);
    }

    public BocloudResult removeTag(String vendor, String region, VolumeModel model) {
        VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.removeTag(model);
    }

	public BocloudResult attach(String vendor, String region, VolumeModel model, String serverId) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.attach(serverId, model);
	}

	public BocloudResult listPolicy(String vendor, String region, String volumeUuid) {
		VolumeProvider provider = getProvider(vendor, VolumeProvider.class, repository, region);
		if (before(provider).isFailed()) {
			return BocloudResult.FAILED("平台认证失败！");
		}
		return provider.listPolicy(volumeUuid);
	}
}
