package com.bocloud.cmp.driver.storage;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.model.VolumePoolModel;
import com.bocloud.cmp.provider.storage.VolumePoolProvider;
import com.bocloud.common.model.BocloudResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 存储池
 * @author luyanchao
 * @since 2019/1/25 10:44
 */
@Service("volumePoolDriver")
public class VolumePoolDriver extends GenericDriver<VolumePoolProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult list(String vendor, String storageUuid) {
        VolumePoolProvider provider = getProvider(vendor, VolumePoolProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.list(storageUuid);
    }

    public BocloudResult listByVendor(String vendor, String region) {
        VolumePoolProvider provider = getProvider(vendor, VolumePoolProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.list();
    }

    public BocloudResult get(String vendor, String volumePoolUuid) {
        VolumePoolProvider provider = getProvider(vendor, VolumePoolProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.get(volumePoolUuid);
    }

    public BocloudResult get(String vendor, String volumePoolUuid, String volumePoolType) {
        VolumePoolProvider provider = getProvider(vendor, VolumePoolProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.get(volumePoolUuid, volumePoolType);
    }

    public BocloudResult create(String vendor, VolumePoolModel model) {
        VolumePoolProvider provider = getProvider(vendor, VolumePoolProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult modify(String vendor, VolumePoolModel model) {
        VolumePoolProvider provider = getProvider(vendor, VolumePoolProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult remove(String vendor, VolumePoolModel model) {
        VolumePoolProvider provider = getProvider(vendor, VolumePoolProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.remove(model);
    }

    public BocloudResult addMember(String vendor, VolumePoolModel model) {
        VolumePoolProvider provider = getProvider(vendor, VolumePoolProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.addMember(model);
    }

    public BocloudResult removeMember(String vendor, VolumePoolModel model) {
        VolumePoolProvider provider = getProvider(vendor, VolumePoolProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.removeMember(model);
    }
}
