package com.bocloud.cmp.driver.veeam;

import com.bocloud.cmp.driver.GenericDriver;
import com.bocloud.cmp.driver.ProviderRepository;
import com.bocloud.cmp.provider.veeam.br.JobProvider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.nms.model.veeam.SessionModel;
import com.bocloud.nms.model.veeam.job.JobCreateModel;
import com.bocloud.nms.model.veeam.job.JobModel;
import com.bocloud.nms.model.veeam.job.JobStateModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service("veeamJobDriver")
public class VeeamJobDriver extends GenericDriver<JobProvider> {

    @Autowired
    private ProviderRepository repository;

    public BocloudResult<JobModel> listById(String id, String vendor){
        JobProvider provider = getProvider(vendor, JobProvider.class, repository);
        if(before(provider).isFailed()){
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.listById(id);
    }

    public BocloudResult<JobStateModel> getJobStateById(String vendor, String id) {
        JobProvider provider = getProvider(vendor, JobProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.getJobStateById(id);
    }

    public BocloudResult<JobModel> create(String vendor, JobCreateModel model) {
        JobProvider provider = getProvider(vendor, JobProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.create(model);
    }

    public BocloudResult<JobModel> modify(String vendor, JobModel model) {
        JobProvider provider = getProvider(vendor, JobProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.modify(model);
    }

    public BocloudResult<SessionModel> startJob(String vendor, String id, boolean performActiveFull, Boolean startChainedJobs) {
        JobProvider provider = getProvider(vendor, JobProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.startJob(id, performActiveFull, startChainedJobs);
    }

    public BocloudResult<SessionModel> stopJob(String vendor, String id, boolean gracefullStop) {
        JobProvider provider = getProvider(vendor, JobProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.stopJob(id, gracefullStop);
    }

    public BocloudResult<Void> removeJob(String vendor, String id) {
        JobProvider provider = getProvider(vendor, JobProvider.class, repository);
        if (before(provider).isFailed()) {
            return BocloudResult.FAILED("平台认证失败！");
        }
        return provider.remove(id);
    }

}
